!----------------------------------------------------------------------
!
!  File:   file_utils.f90
!  Author: Ian Ross, University of Bristol
!  Date:   19 Feb 2005
!
!----------------------------------------------------------------------
!
!  File utility routines for FOAM model-specific code.
!
!----------------------------------------------------------------------

MODULE file_utils

  USE pp_defs
  USE pp_utils
  USE netcdf

  USE foam_defs

  IMPLICIT NONE

  INTERFACE read_2d_netcdf_var
     MODULE PROCEDURE read_2d_netcdf_var_r
     MODULE PROCEDURE read_2d_netcdf_var_i
  END INTERFACE

  PRIVATE
  PUBLIC :: open_history_file, open_aux_file
  PUBLIC :: close_file
  PUBLIC :: normalise_dir
  PUBLIC :: read_0d_netcdf_var, read_1d_netcdf_var
  PUBLIC :: read_2d_netcdf_var, read_2d_netcdf_slice, read_3d_netcdf_var
  PUBLIC :: foam_var_name

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Locate and open the appropriate FOAM history file for a given
  !  variable on a given day.
  !

  FUNCTION open_history_file(var, day) RESULT(nc_id)

    IMPLICIT NONE

    CHARACTER(LEN=*), INTENT(IN) :: var    ! Variable name.
    INTEGER,          INTENT(IN) :: day    ! History day.

    INTEGER                      :: nc_id  ! netCDF ID for file.


    CHARACTER(LEN=1024) :: file_name         ! File name.
    INTEGER :: status                        ! netCDF file open status.


    ! Construct the filename from the foam data directory path, the
    ! directory relevant for the variable and the file name relevant for
    ! the history day.

    file_name = TRIM(foam_data_dir) // &
         TRIM(history_dir(var)) // '/' // TRIM(history_file(day))
    WRITE (*, 1) TRIM(file_name)
1   FORMAT ('Opening FOAM output file: ', A)


    ! Open the file.

    status = NF90_OPEN(file_name, NF90_NOWRITE, nc_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_OPEN, TRIM(history_dir(var)) // &
         '/' // TRIM(history_file(day)))

  END FUNCTION open_history_file


  !--------------------------------------------------------------------
  !
  !  Open an auxiliary netCDF file.
  !

  FUNCTION open_aux_file(file_name) RESULT(nc_id)

    IMPLICIT NONE

    CHARACTER(LEN=*), INTENT(IN) :: file_name ! File name.

    INTEGER                      :: nc_id     ! netCDF ID for file.


    INTEGER :: status                        ! netCDF file open status.


    ! Open the file.

    status = NF90_OPEN(file_name, NF90_NOWRITE, nc_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_OPEN, file_name)

  END FUNCTION open_aux_file


  !--------------------------------------------------------------------
  !
  !  Close an open FOAM history file.
  !

  SUBROUTINE close_file(nc_id)

    IMPLICIT NONE

    INTEGER, INTENT(IN) :: nc_id  ! netCDF ID for file.


    INTEGER :: status           ! netCDF file close status.


    ! Close the file.

    status = NF90_CLOSE(nc_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_CLOSE)

  END SUBROUTINE close_file


  !--------------------------------------------------------------------
  !
  !  Read a scale netCDF variable.
  !

  SUBROUTINE read_0d_netcdf_var(nc_id, var, data)

    IMPLICIT NONE

    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.

    REAL,                        INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id

    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))

    status = NF90_GET_VAR(nc_id, var_id, data)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))

  END SUBROUTINE read_0d_netcdf_var


  !--------------------------------------------------------------------
  !
  !  Read a 1-D netCDF variable: assumes that the output data parameter is
  !  the right size!
  !

  SUBROUTINE read_1d_netcdf_var(nc_id, var, data)

    IMPLICIT NONE

    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.

    REAL, DIMENSION(:),          INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id

    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))

    status = NF90_GET_VAR(nc_id, var_id, data)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))

  END SUBROUTINE read_1d_netcdf_var


  !--------------------------------------------------------------------
  !
  !  Read a 2-D netCDF variable: assumes that the output data parameter is
  !  the right size!
  !

  SUBROUTINE read_2d_netcdf_var_r(nc_id, var, data)

    IMPLICIT NONE

    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.

    REAL, DIMENSION(:,:),        INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id

    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))

    status = NF90_GET_VAR(nc_id, var_id, data)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))

  END SUBROUTINE read_2d_netcdf_var_r

  SUBROUTINE read_2d_netcdf_var_i(nc_id, var, data)

    IMPLICIT NONE

    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.

    INTEGER, DIMENSION(:,:),     INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id

    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))

    status = NF90_GET_VAR(nc_id, var_id, data)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))

  END SUBROUTINE read_2d_netcdf_var_i


  !--------------------------------------------------------------------
  !
  !  Read a 2-D slice of a 3-D netCDF variable: assumes that the output
  !  data parameter is the right size!
  !

  SUBROUTINE read_2d_netcdf_slice(nc_id, var, slice, data)

    IMPLICIT NONE

    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.
    INTEGER,                     INTENT(IN)  :: slice  ! Position of slice.

    REAL, DIMENSION(:,:),        INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id
    INTEGER, DIMENSION(3) :: start, count

    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))

    start = (/ 1, 1, slice /)
    count = (/ SIZE(data, 1), SIZE(data, 2), 1 /)
    status = NF90_GET_VAR(nc_id, var_id, data, start, count)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))

  END SUBROUTINE read_2d_netcdf_slice


  !--------------------------------------------------------------------
  !
  !  Read a 3-D netCDF variable: assumes that the output data parameter is
  !  the right size!
  !

  SUBROUTINE read_3d_netcdf_var(nc_id, var, data)

    IMPLICIT NONE

    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.

    REAL, DIMENSION(:,:,:),      INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id

    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))

    status = NF90_GET_VAR(nc_id, var_id, data)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))

  END SUBROUTINE read_3d_netcdf_var


  !--------------------------------------------------------------------
  !
  !  Normalise a directory name so that it has a slash as its last
  !  character.
  !

  SUBROUTINE normalise_dir(dir)

    IMPLICIT NONE

    CHARACTER(LEN=*), INTENT(INOUT) :: dir

    INTEGER :: last

    last = LEN_TRIM(dir)
    IF (dir(last:last) /= '/') dir = TRIM(dir) // '/'

  END SUBROUTINE normalise_dir


  !--------------------------------------------------------------------
  !
  !  Determine the sub-directory within the FOAM output directory for a
  !  given variable.  This also checks that only variables that are
  !  actually contained in FOAM history files are read.
  !

  FUNCTION history_dir(var) RESULT(dirname)

    IMPLICIT NONE

    CHARACTER(LEN=*), INTENT(IN) :: var

    CHARACTER(LEN=ARG_LEN)       :: dirname


    SELECT CASE (var)
    CASE ('ta', 'ts', 'prc', 'prl', 'cl', 'clt', 'ps', 'hus', 'ua', 'va',  &
          'wap', 'rss', 'rsscs', 'rsdt', 'rst', 'rstcs', 'rls', 'rlscs',   &
          'rlut', 'rlutcs', 'hur', 'zg', 'psl', 'albs', 'rsds', 'rsus',    &
          'rsut', 'rsdscs', 'rsuscs', 'rsutcs', 'rlds', 'rlus', 'rldscs',  &
          'tas', 'tasmin', 'tasmax', 'huss', 'hurs', 'uas', 'vas',         &
          'evspsbl', 'prw', 'clivi', 'clwvi', 'snc', 'snd', 'snm',         &
          'intuaw', 'intvaw', 'intuadse', 'intvadse', 'hfss', 'hfls', 'sbls')
       dirname = 'atmos'

    CASE ('uo', 'vo', 'wo', 'thetao', 'so')
       dirname = 'ocean'

    CASE ('tsoil1', 'tsoil2', 'tsoil3', 'tsoil4', 'sit', 'siprod', &
          'mrso', 'hfo', 'wfo', 'tauu', 'tauv', 'mrro', 'dis', 'sic')
       dirname = 'coupl'

    CASE ('vcarpft', 'scar', 'pft', 'npppft', 'laipft', 'bmb', 'lcarpft', &
          'cpyh', 'gpppft', 'transpft', 'esoil', 'potevpft', 'hresp')
       dirname = 'lpj'

    CASE DEFAULT
       CALL fatal(ERR_INTERNAL, 'bad variable for FOAM file: ' // TRIM(var))
    END SELECT

  END FUNCTION history_dir


  !--------------------------------------------------------------------
  !
  !  Generate the history file name for a given day.
  !

  FUNCTION history_file(day) RESULT(fname)

    IMPLICIT NONE

    INTEGER, INTENT(IN)    :: day

    CHARACTER(LEN=ARG_LEN) :: fname

    CHARACTER(LEN=ARG_LEN) :: tmp


    WRITE (tmp, '(I8)') 10000000 + day
    fname = tmp(2:8) // '.nc'

  END FUNCTION history_file


  !--------------------------------------------------------------------
  !
  !  Map MOTIF variable names to FOAM ones.  Most are the same.
  !

  FUNCTION foam_var_name(motif_name) RESULT(foam_name)

    IMPLICIT NONE

    CHARACTER(LEN=*), INTENT(IN) :: motif_name

    CHARACTER(LEN=VAR_NAME_LEN)  :: foam_name


    SELECT CASE (motif_name)
    CASE ('vcarcont')
       foam_name = 'vcar'

    CASE ('vcarcontpft')
       foam_name = 'vcarpft'

    CASE ('scarcont')
       foam_name = 'scar'

    CASE ('pftfrac')
       foam_name = 'pft'

    CASE ('bioburnt')
       foam_name = 'bmb'

    CASE ('lcarcont')
       foam_name = 'lcar'

    CASE ('lcarcontpft')
       foam_name = 'lcarpft'

    CASE ('cheight')
       foam_name = 'cpyh'

    CASE ('potevap')
       foam_name = 'potev'

    CASE ('potevappft')
       foam_name = 'potevpft'

    CASE DEFAULT
       foam_name = TRIM(motif_name)
    END SELECT

  END FUNCTION foam_var_name

END MODULE file_utils

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
