!----------------------------------------------------------------------
!
!  File:   model_main.f90
!  Author: Ian Ross, University of Bristol
!  Date:   17 Feb 2005
!
!----------------------------------------------------------------------
!
!  Model-specific code for MOTIF/PMIP2 post-processor for FOAM/LPJ
!  model.
!
!  There are five routines that the model-specific code needs to
!  support:
!
!-----------------------------------------------------------------------
!  model_init(cfg_args, cfg_entries, description, cal_info, missing)   !
!                                                                      !
!  TYPE(cfg_arg), DIMENSION(MAX_CFG_ARGS), INTENT(IN)  :: cfg_args     !
!  INTEGER,                                INTENT(IN)  :: cfg_entries  !
!  CHARACTER(LEN=ARG_LEN),                 INTENT(OUT) :: description  !
!  TYPE(calendar_info),                    INTENT(OUT) :: cal_info     !
!  REAL,                                   INTENT(OUT) :: missing      !
!-----------------------------------------------------------------------
!
!    Initialise the model-specific code.  The information read from the
!    configuration file is passed in in the cfg_args and cfg_entries
!    arguments - this information should be used to set up any internal
!    information needed by the model-specific code to function. 
!    Additionally, the description and cal_info arguments should be
!    filled in with a textual description of the model and information
!    about the calendar used, and missing should be set to the missing
!    value used in the model data.
!
!----------------------------------------------------------------------
!  model_get_axis(axis, var_name)                 !
!                                                 !
!  TYPE(axis_def), INTENT(INOUT)      :: axis     !
!  CHARACTER(LEN=ARG_LEN), INTENT(IN) :: var_name !
!--------------------------------------------------
!
!   Fill in the definition for a named axis appropriate to the
!   specific model.  The axis_def object passed in has the axis name
!   filled in, but information about the values and range of the axis
!   must be completed by the model_get_axis implementation.  The
!   possible values of the name field in the axis_def object are
!   'longitude', 'latitude', 'depth' (meaning either ocean depth or
!   soil depth), and 'pft'.  The domain (atmosphere, ocean, surface,
!   ice) is determined by the domain member of axis, and should be
!   used to distinguish between, for instance, different latitude/longitude
!   grids used for the atmosphere and ocean.  The variable name is passed
!   in for convenience for models that use different axes for different
!   variables in the same domain (e.g. staggered grids).
!
!----------------------------------------------------------------------
!  FUNCTION model_get_units(var_name)                          !
!                                                              !
!  CHARACTER(LEN=UNIT_NAME_LEN)            :: model_get_units  !
!  CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN) :: var_name         !
!---------------------------------------------------------------
!
!   Return the units used by the model (as a udunits compatible unit
!   string) for variable var_name.
!
!----------------------------------------------------------------------
!  FUNCTION model_get_positive(var_name)                     !
!                                                            !
!  CHARACTER(LEN=ARG_LEN)             :: model_get_positive  !
!  CHARACTER(LEN=ARG_LEN), INTENT(IN) :: var_name            !
!-------------------------------------------------------------
!
!   Return the positive direction ('up' or 'down') for a given flux
!   variable.
!
!----------------------------------------------------------------------
!  model_get_value(var, sday, eday, data)   ! <--- Single values
!                                           !
!  TYPE(variable_def), INTENT(IN)  :: var   !
!  REAL,               INTENT(IN)  :: sday  !
!  REAL,               INTENT(IN)  :: eday  !
!  REAL,               INTENT(OUT) :: data  !
!------------------------------------------------------
!  model_get_value(var, sday, eday, axes, data)       ! <--- Hemispheric values
!                                                     !
!  TYPE(variable_def),           INTENT(IN)  :: var   !
!  REAL,                         INTENT(IN)  :: sday  !
!  REAL,                         INTENT(IN)  :: eday  !
!  TYPE(axis_def), DIMENSION(1), INTENT(IN)  :: axes  !
!  REAL, DIMENSION(2),           INTENT(OUT) :: data  !
!------------------------------------------------------
!  model_get_value(var, sday, eday, axes, data)       ! <--- 2-D data
!                                                     !
!  TYPE(variable_def),           INTENT(IN)  :: var   !
!  REAL,                         INTENT(IN)  :: sday  !
!  REAL,                         INTENT(IN)  :: eday  !
!  TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes  !
!  REAL, DIMENSION(:, :),        INTENT(OUT) :: data  !
!------------------------------------------------------
!  model_get_value(var, sday, eday, axes, data)       ! <--- 3-D data
!                                                     !
!  TYPE(variable_def),           INTENT(IN)  :: var   !
!  REAL,                         INTENT(IN)  :: sday  !
!  REAL,                         INTENT(IN)  :: eday  !
!  TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes  !
!  REAL, DIMENSION(:, :, :),     INTENT(OUT) :: data  !
!------------------------------------------------------
!  
!   Set up data array or data value for a given variable for a single
!   timestep.  Four different calling sequences must be supported by
!   the model_get_value interface: one for single value variables
!   (for instance, sifram), one for hemispheric values (for instance,
!   siarea), one for 2-D data, and one for 3-D data.  2- and 3-D data
!   can be defined on any of the possible axes, not just the obvious
!   long./lat./pressure grid - for instance, XYV data is 3-D, YRT
!   data is 2-D, and so on.
!
!----------------------------------------------------------------------

MODULE model_main

  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils

  USE foam_defs                 ! FOAM/LPJ specific definitions.
  USE vars_2d
  USE vars_3d
  USE ice_vars
  USE file_utils
  USE math_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: model_init
  PUBLIC :: model_get_axis
  PUBLIC :: model_get_units
  PUBLIC :: model_get_positive
  PUBLIC :: model_get_value

  INTERFACE model_get_value
     MODULE PROCEDURE get_val_single ! 1T
     MODULE PROCEDURE get_val_1d     ! 2HT
     MODULE PROCEDURE get_val_2d     ! XY, XYT, YRT
     MODULE PROCEDURE get_val_3d     ! XYPT, XYST, XYVT, XYZT, YZRT
  END INTERFACE

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Initialise the FOAM/LPJ model interface.  Model-specific command
  !  line arguments are available in the command_line array.
  !

  SUBROUTINE model_init(cfg_args, cfg_entries, description, cal_info, missing)

    IMPLICIT NONE

    TYPE(cfg_arg), DIMENSION(MAX_CFG_ARGS), INTENT(IN)  :: cfg_args
    INTEGER,                                INTENT(IN)  :: cfg_entries

    CHARACTER(LEN=ARG_LEN),                 INTENT(OUT) :: description
    TYPE(calendar_info),                    INTENT(OUT) :: cal_info
    REAL,                                   INTENT(OUT) :: missing


    ! Set up model description and calendar information.

    description = 'FOAM/LPJ 1.5 (2004) ' // &
         'atmosphere: CCM2 (R15L18); ' // &
         'ocean: OM3 (128x128L24); ' // &
         'sea ice: CSMICE; land: LPJ'
    cal_info%name = '360_day'
    cal_info%mon_lens = (/ 30, 30, 30, 30, 30, 30, &
                           30, 30, 30, 30, 30, 30  /)
    missing = FOAM_MISSING_VAL


    ! Extract mandatory configuration entries for FOAM/LPJ.

    foam_data_dir = get_cfg_value(cfg_args, cfg_entries, 'foam_data_dir')
    IF (TRIM(foam_data_dir) == '') CALL fatal(ERR_CFG_MISSING, 'foam_data_dir')
    CALL normalise_dir(foam_data_dir)


    ! Extract optional information from the configuration file
    ! (foam_fixed_dir is only needed to get things like the orography, so
    ! it's not mandatory).

    foam_fixed_dir = get_cfg_value(cfg_args, cfg_entries, 'foam_fixed_dir')
    IF (TRIM(foam_data_dir) /= '') CALL normalise_dir(foam_fixed_dir)
    regions_file = get_cfg_value(cfg_args, cfg_entries, 'foam_regions')

  END SUBROUTINE model_init


  !--------------------------------------------------------------------
  !
  !  Fill in a given axis definition object from the axis name and
  !  domain.  Since FOAM uses the same latitude grid for ocean,
  !  surface and ice models, there are only two possible results for
  !  latitude and longitude axes.  Ocean and soil depth axes are
  !  treated correctly, as is the PFT axis.
  !
  !  NOTE: THE cell_bounds INFORMATION *MUST* BE SUPPLIED FOR LONGITUDE
  !        AND LATITUDE AXES.  CMOR WILL CRASH IF YOU DON'T SUPPLY THIS
  !        INFORMATION!
  !

  SUBROUTINE model_get_axis(axis, var_name)

    IMPLICIT NONE

    ! The domain and name elements of the axis definition are set up
    ! on entry, while the values entry is unallocated.  If the domain
    ! and name are appropriate, model_get_axis will allocate and fill
    ! the values entry.  The variable name is passed in for convenience
    ! for models that use different axes for different variables in the
    ! same domain (e.g. staggered grids).

    TYPE(axis_def), INTENT(INOUT)      :: axis
    CHARACTER(LEN=ARG_LEN), INTENT(IN) :: var_name ! Variable name.

    INTEGER :: i


    SELECT CASE (axis%name)

       ! Latitude and longitude axes.
       
    CASE (AXIS_LON, AXIS_LAT)
       CALL lat_lon_grid(axis%domain, axis%name, axis%values, axis%bounds)


       ! The ocean model has 24 layers and the soil model soil model
       ! has 4 layers.

    CASE (AXIS_DEPTH)
       SELECT CASE (axis%domain)
       CASE (DOMAIN_OCEAN)
          ALLOCATE(axis%values(FOAM_ODPTH))
          axis%values = &
               (/ 5300.0, 4900.0, 4500.0, 4100.0, 3700.0, &
                  3300.0, 2900.0, 2500.0, 2100.0, 1700.0, &
                  1307.5,  952.5,  675.0,  484.0,  357.0, &
                   271.0,  210.0,  165.0,  129.0,   99.5, &
                    74.0,   51.0,   30.0,   10.0 /)

       CASE (DOMAIN_SURFACE)
          ALLOCATE(axis%values(FOAM_SDPTH))
          axis%values = (/ 0.12, 0.44, 1.36, 8.13 /)
       END SELECT


       ! Nine PFTs are defined in the LPJ vegetation model.

    CASE (AXIS_PFT)
       ALLOCATE(axis%values(FOAM_NPFT))
       axis%values = (/ 1, 2, 3, 4, 5, 6, 7, 8, 9 /)


       ! Scalar height marker axes.

    CASE (AXIS_HGT1, AXIS_HGT2)
       ALLOCATE(axis%values(1))
       IF (TRIM(axis%name) == AXIS_HGT1) THEN
          axis%values(1) = 2.0
       ELSE
          axis%values(1) = 10.0
       END IF


       ! Other axes should be handled by the main program.

    CASE DEFAULT
       CALL fatal(ERR_UNKNOWN_AXIS, axis%name)

    END SELECT

  END SUBROUTINE model_get_axis


  !--------------------------------------------------------------------
  !
  !  Return unit used by model for a given variable.  All of the units
  !  used in FOAM output are identical to those required in the MOTIF
  !  output files.
  !

  FUNCTION model_get_units(var_name) RESULT(units)

    CHARACTER(LEN=UNIT_NAME_LEN)            :: units
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN) :: var_name
    

    SELECT CASE (var_name)

       CASE ('laipft', 'albs', 'fpar', 'pftfrac')
          units = '1'

       CASE ('sftgif', 'sftlf', 'clt', 'sic', 'snc', 'cl')
          units = '1'

       CASE ('hurs', 'hur')
          units = '0.01'

       CASE ('intuadse', 'intvadse')
          units = '1.0E6 J m-1 s-1'

       CASE ('intuaw', 'intvaw')
          units = '1.0E6 kg m-1 s-1'

       CASE ('so', 'sos', 'sozm')
          units = '1.0E-3'

       CASE ('tasmax', 'tasmin', 'tsoil', 'tas', 'ta', 'ts')
          units = 'K'

       CASE ('thetaozm', 'thetao', 'tos')
          units = 'degC'

       CASE ('hus', 'huss')
          units = 'kg kg-1'

       CASE ('prw', 'mrso', 'clivi', 'clwvi')
          units = 'kg m-2'

       CASE ('bioburnt')
          units = 'gram m-2 month-1'

       CASE ('lcarcont', 'scarcont', 'vcarcont', 'lcarcontpft', 'vcarcontpft')
          units = 'gram m-2'

       CASE ('pr', 'prc')
          units = '1000 kg m-2 s-1'

       CASE ('snm', 'wfo', 'mrro', 'sbls', 'esoil',   &
             'mrros', 'potevap', 'trans', 'evspsbl', 'potevappft', &
             'transpft', 'evspsblveg', 'hresp')
          units = 'kg m-2 s-1'

       CASE ('gpp', 'npp', 'gpppft', 'npppft')
          units = 'gram m-2 s-1'

       CASE ('sltov')
          units = 'kg s-1'

       CASE ('sildarea', 'siarea', 'siext')
          units = 'km2'

       CASE ('sisnvol', 'sivol')
          units = 'km3'

       CASE ('ziso20', 'cheight', 'orog', 'zmlo', 'sit', 'snd', 'zos', 'zg')
          units = 'm'

       CASE ('dis', 'stfmmc', 'stfbarot')
          units = 'm3 s-1'

       CASE ('hcont300')
          units = 'm K'

       CASE ('siprod')
          units = 'm month-1'

       CASE ('ua', 'uo', 'va', 'vo', 'wo', 'uas', 'uos', 'usi', 'vas', &
             'vos', 'vsi', 'uvas')
          units = 'm s-1'

       CASE ('mpuoto', 'mpvoto', 'mpuoeito', 'mpvoeito')
          units = 'm s-1 K'

       CASE ('mpuoso', 'mpvoso', 'mpuoeiso', 'mpvoeiso')
          units = 'm s-1 1.0E-3'

       CASE ('tauu', 'tauv', 'psl', 'ps')
          units = 'Pa'

       CASE ('wap')
          units = 'Pa s-1'

       CASE ('sifram')
          units = '1.0E6 m3 s-1'

       CASE ('hfogo')
          units = 'W'

       CASE ('hfo', 'rls', 'rss', 'rst', 'hfls', 'hfns', 'hfss', 'rlds', &
             'rlus', 'rlut', 'rsds', 'rsdt', 'rsus', 'rsut', 'rtmt',     &
             'hfsib', 'rlscs', 'rsscs', 'rstcs', 'rldscs', 'rlutcs',     &
             'rsdscs', 'rsuscs', 'rsutcs')
          units = 'W m-2'

       END SELECT

  END FUNCTION model_get_units


  !--------------------------------------------------------------------
  !
  !  Return positive direction for a flux variable.
  !

  FUNCTION model_get_positive(var_name) RESULT(dir)

    CHARACTER(LEN=ARG_LEN)             :: dir
    CHARACTER(LEN=ARG_LEN), INTENT(IN) :: var_name
    

    ! For FOAM, they're all downward apart from the radiation fluxes
    ! explicitly labelled as upwards, and the latent and sensible heat
    ! fluxes.

    SELECT CASE (TRIM(var_name))

    CASE ('rlus', 'rlut', 'rsus', 'rsut', 'rlutcs', 'rsuscs', 'rsutcs', &
          'hfls', 'hfss')
       dir = 'up'

    CASE DEFAULT
       dir = 'down'

    END SELECT

  END FUNCTION model_get_positive


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for singleton data.
  !

  SUBROUTINE get_val_single(var, sday, eday, data)

    IMPLICIT NONE

    TYPE(variable_def), INTENT(IN)  :: var
    INTEGER,            INTENT(IN)  :: sday, eday

    REAL,               INTENT(OUT) :: data


    ! FOAM doesn't handle any of these.

    data = 0
    CALL fatal(ERR_MODEL_INVALID_VAR, var%name)

  END SUBROUTINE get_val_single


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 1-D data.
  !

  SUBROUTINE get_val_1d(var, sday, eday, axes, data)

    IMPLICIT NONE

    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(1), INTENT(IN)  :: axes

    REAL, DIMENSION(:),           INTENT(OUT) :: data


    SELECT CASE (var%name)
    CASE ('siarea')
       CALL get_sea_ice_area(sday, eday, data)

    CASE ('siext')
       CALL get_sea_ice_extent(sday, eday, data)

    CASE ('sivol')
       CALL get_sea_ice_volume(sday, eday, data)

    CASE DEFAULT
       CALL fatal(ERR_MODEL_INVALID_VAR, var%name)
    END SELECT

  END SUBROUTINE get_val_1d


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 2-D data.
  !

  SUBROUTINE get_val_2d(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, averarging day range and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    ! Temporary data storage.
    REAL, DIMENSION(SIZE(data, 1), SIZE(data, 2)) :: tmp_data

    ! Loop counters and temporaries.
    INTEGER :: day, slice_len, slices

    data = 0
    slice_len = foam_storage_freq(var)
    slices = 0

    DO day = sday, eday, slice_len
       CALL get_one_slice_2d(var, day, day + slice_len - 1, axes, tmp_data)
       IF (slices == 0) THEN
          data = tmp_data
       ELSE
          WHERE (tmp_data /= FOAM_MISSING_VAL .AND. &
               data /= FOAM_MISSING_VAL)
             data = data + tmp_data
          ELSEWHERE
             data = FOAM_MISSING_VAL
          END WHERE
       END IF
       slices = slices + 1
    END DO
    WHERE (data /= FOAM_MISSING_VAL) data = data / slices

  END SUBROUTINE get_val_2d


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 3-D data.
  !

  SUBROUTINE get_val_3d(var, sday, eday, axes, data)

    IMPLICIT NONE

    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes

    REAL, DIMENSION(:, :, :),     INTENT(OUT) :: data


    ! Temporary data storage.
    REAL, DIMENSION(SIZE(data, 1), SIZE(data, 2), SIZE(data, 3)) :: tmp_data

    ! Loop counters and temporaries.
    INTEGER :: day, slice_len, slices


    ! Accumulate data, allowing missing values from the single slice
    ! extraction to contaminate the accumulated data.  This is needed
    ! because when we regrid onto the required pressure levels, there are
    ! some places where missing data is generated simply because some of
    ! the levels are below the surface in some locations.  This needs to
    ! be handled properly here.

    data = 0
    slice_len = foam_storage_freq(var)
    slices = 0

    DO day = sday, eday, slice_len
       CALL get_one_slice_3d(var, day, day + slice_len - 1, axes, tmp_data)
       WHERE (tmp_data == FOAM_MISSING_VAL) data = FOAM_MISSING_VAL
       WHERE (data /= FOAM_MISSING_VAL)     data = data + tmp_data
       slices = slices + 1
    END DO


    ! Average the data over the time range, dealing with missing values.

    WHERE (data /= FOAM_MISSING_VAL) data = data / slices

  END SUBROUTINE get_val_3d


  !--------------------------------------------------------------------
  !
  !  Return frequency of storage of variable in FOAM.  Used to determine
  !  averaging intervals above.
  !

  FUNCTION foam_storage_freq(var) RESULT(freq)

    TYPE(variable_def), INTENT(IN)  :: var

    INTEGER :: freq
    

    SELECT CASE (var%domain)
    CASE (DOMAIN_ATMOS, DOMAIN_ICE)
       IF (TRIM(var%name) == 'hfsib') THEN
          freq = 30
       ELSE
          freq = 1
       END IF

    CASE (DOMAIN_OCEAN)
       freq = 30

    CASE (DOMAIN_SURFACE)
       SELECT CASE (var%name)
       CASE ('tsoil', 'sit', 'siprod', 'mrso', 'hfo', 'wfo', &
             'tauu', 'tauv', 'mrro', 'dis', 'sic')
          freq = 1

       CASE DEFAULT
          freq = 30
       END SELECT
    END SELECT

  END FUNCTION foam_storage_freq

END MODULE model_main

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
