!----------------------------------------------------------------------
!
!  File:   ocean_utils.f90
!  Author: Ian Ross, University of Bristol
!  Date:   22 Feb 2005
!
!----------------------------------------------------------------------
!
!  Utility routines for calculating some ocean fields for FOAM
!  model-specific code.
!
!----------------------------------------------------------------------

MODULE ocean_utils

  USE pp_defs
  USE pp_utils
  USE foam_defs
  USE file_utils
  USE math_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: calc_ocean_mixed_layer
  PUBLIC :: calc_stfmmc
  PUBLIC :: eos
  PUBLIC :: point_is_in

CONTAINS


  !--------------------------------------------------------------------
  !
  !  Calculate the FOAM meridional streamfunction.
  !

  SUBROUTINE calc_stfmmc(vo, stfmmc)

    IMPLICIT NONE

    ! Meridional velocity field.
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH), INTENT(IN)  :: vo

    ! Meridional streamfunction (per region, plus global).
    REAL, DIMENSION(FOAM_OLAT, FOAM_ODPTH, 4),         INTENT(OUT) :: stfmmc


    ! Longitude/latitude grid.
    REAL, DIMENSION(:), ALLOCATABLE, SAVE    :: lat_vals
    REAL, DIMENSION(:, :), ALLOCATABLE, SAVE :: lat_bnds
    LOGICAL, SAVE                            :: first = .TRUE.

    ! Ocean level depths and thicknesses.
    REAL, PARAMETER, DIMENSION(FOAM_ODPTH) :: levs = &
         (/ 5300.0, 4900.0, 4500.0, 4100.0, 3700.0, 3300.0, 2900.0, 2500.0, &
            2100.0, 1700.0, 1307.5,  952.5,  675.0,  484.0,  357.0,  271.0, &
             210.0,  165.0,  129.0,   99.5,   74.0,   51.0,   30.0,   10.0  /)
    REAL, PARAMETER, DIMENSION(FOAM_ODPTH) :: thick = &
         (/ 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, 400.0, &
            400.0, 400.0, 385.0, 325.0, 230.0, 152.0, 102.0,  70.0, &
             51.0,  40.0,  32.0,  27.0,  24.0,  22.0,  20.0,  20.0  /)


    ! Zonally integrated meridional velocity (units m^2/s).
    REAL, DIMENSION(FOAM_OLAT, FOAM_ODPTH, 4) :: vint

    ! Region mask for regional MOC calculation.
    LOGICAL, DIMENSION(FOAM_OLAT, FOAM_ODPTH, 4) :: mask

    ! Temporary values and loop counters.
    INTEGER :: ix, iy, iz, region
    REAL :: ds_lon


    ! Get latitude values and bounds.

    IF (first) THEN
       CALL lat_lon_grid(DOMAIN_OCEAN, AXIS_LAT, lat_vals, lat_bnds)
       first = .FALSE.
    END IF


    ! Zonal velocity integration (by region and globally).

    vint = 0.0
    mask = .FALSE.
    DO iz = 1, FOAM_ODPTH
       DO iy = 1, FOAM_OLAT
          DO ix = 1, FOAM_OLON
             IF (vo(ix, iy, iz) == FOAM_MISSING_VAL) CYCLE

             DO region = REGION_ATLANTIC, REGION_PACIFIC
                IF (point_is_in(region, ix, iy)) THEN
                   mask(iy, iz, region) = .TRUE.
                   vint(iy, iz, region) = vint(iy, iz, region) + vo(ix, iy, iz)
                END IF
             END DO

             vint(iy, iz, REGION_GLOBAL) = &
                  vint(iy, iz, REGION_GLOBAL) + vo(ix, iy, iz)
             mask(iy, iz, REGION_GLOBAL) = .TRUE.
          END DO

          ! Correct by the distance factor: ds_lon is the distance along the
          ! relevant latitude circle for one longitude step.

          ds_lon = 2.0 * PI * (R_EARTH * 1.0E3 - levs(iz)) * &
               COS(D_TO_R * lat_vals(iy)) / FOAM_OLON
          vint(iy, iz, :) = vint(iy, iz, :) * ds_lon
       END DO
    END DO


    ! Integrate from top of ocean downwards - the stream function at a
    ! point P is defined as \psi(P) = \int_O^P v dz - w dy, with O a
    ! reference point, and is independent of the integration path.  If the
    ! reference point is taken as being any point on the surface, and the
    ! integral path runs across the surface until vertically above point P,
    ! then vertically downwards, the integral can be evaluated as a simple
    ! sum of the zonally integrated meridional velocity (using the layer
    ! thicknesses as weights in the sum).
    !
    ! Be careful here: levels run from 1 at the bottom to FOAM_ODPTH at
    ! the top!

    DO iy = 1, FOAM_OLAT
       DO iz = FOAM_ODPTH, 1, -1
          DO region = REGION_ATLANTIC, REGION_GLOBAL
             ! Just copy missing values over - once we get one missing
             ! values, all the others until the bottom of the ocean will
             ! also be missing.

             IF (vint(iy, iz, region) == FOAM_MISSING_VAL .OR. &
                  .NOT. mask(iy, iz, region)) THEN
                stfmmc(iy, iz, region) = FOAM_MISSING_VAL
             ELSE
                IF (iz == FOAM_ODPTH) THEN
                   stfmmc(iy, iz, region) = vint(iy, iz, region) * thick(iz)
                ELSE
                   stfmmc(iy, iz, region) = stfmmc(iy, iz + 1, region) + &
                        vint(iy, iz, region) * thick(iz)
                END IF
             END IF
          END DO
       END DO
    END DO

  END SUBROUTINE calc_stfmmc


  !--------------------------------------------------------------------
  !
  !  Calculate the FOAM ocean mixed layer depth output.
  !

  SUBROUTINE calc_ocean_mixed_layer(t, s, mld)

    IMPLICIT NONE

    ! 3-D temperature and salinity field inputs.
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH), INTENT(IN)  :: t
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH), INTENT(IN)  :: s

    ! 2-D mixed layer depth output.
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT),             INTENT(OUT) :: mld


    ! Ocean level depths.
    REAL, DIMENSION(FOAM_ODPTH) :: z = &
         (/ 5300.0, 4900.0, 4500.0, 4100.0, 3700.0, &
            3300.0, 2900.0, 2500.0, 2100.0, 1700.0, &
            1307.5,  952.5,  675.0,  484.0,  357.0, &
             271.0,  210.0,  165.0,  129.0,   99.5, &
              74.0,   51.0,   30.0,   10.0 /)

    ! Density profile.
    REAL, DIMENSION(FOAM_ODPTH) :: rho


    INTEGER :: ix, iy, iz, kmax

    DO ix = 1, SIZE(t, 1)
       DO iy = 1, SIZE(t, 2)
          IF (ALL(t(ix, iy, :) == FOAM_MISSING_VAL)) THEN
             mld(ix, iy) = FOAM_MISSING_VAL
          ELSE
             DO kmax = 1, FOAM_ODPTH
                IF (t(ix, iy, kmax) /= FOAM_MISSING_VAL) EXIT
             END DO
             IF (kmax > FOAM_ODPTH) kmax = FOAM_ODPTH
             rho = FOAM_MISSING_VAL
             DO iz = kmax, FOAM_ODPTH
                rho(iz) = eos(t(ix, iy, iz), s(ix, iy, iz), iz)
             END DO
             CALL mld_from_density(t(ix, iy, :), s(ix, iy, :), rho, kmax, &
                                   z, mld(ix, iy))
          END IF
       END DO
    END DO

  END SUBROUTINE calc_ocean_mixed_layer


  !--------------------------------------------------------------------
  !
  !  FOAM ocean model equation of state.
  !

  FUNCTION eos(t, s, iz) RESULT(rho)

    IMPLICIT NONE

    REAL, INTENT(IN)    :: t
    REAL, INTENT(IN)    :: s
    INTEGER, INTENT(IN) :: iz
    REAL                :: rho


    ! Reference density (as sigma t value).

    REAL, PARAMETER :: sigmat_ref = 30.0


    ! Normalized temperatures, salinities and coefficients generated by POP
    ! program "denscoef" which fits 3rd order polynomials to the equation of
    ! state for each model level.

    REAL, DIMENSION(FOAM_ODPTH) :: to = &
         (/ 3.3557111,  3.3535789,  2.8750204,  2.8746644, &
            2.8763988,  2.8802727,  2.8863349,  2.8946335, &
            2.9052159,  2.9181287,  2.9330032,  4.4374887, &
            5.9464594,  8.4521018,  8.4643947,  8.4728308, &
           13.9680775, 13.9749415, 13.9803867, 13.9848580, &
           13.9887297, 13.9922271, 13.9954247, 13.9984740  /)

    REAL, DIMENSION(FOAM_ODPTH) :: so = &
         (/ -0.0002000, -0.0002000, -0.0002000, -0.0002000, &
            -0.0002000, -0.0002000, -0.0002000, -0.0002000, &
            -0.0002000, -0.0002000, -0.0002500, -0.0001000, &
            -0.0001000,  0.0001500,  0.0001500,  0.0001500, &
            -0.0092500, -0.0092500, -0.0092500, -0.0092500, &
            -0.0092500, -0.0092500, -0.0092500, -0.0092500  /)

    REAL, DIMENSION(9, FOAM_ODPTH) :: c = RESHAPE( &
    shape=(/ 9, FOAM_ODPTH /),  &
  source=(/ -.2314274E-03, 0.7602750E+00, -.4055873E-05, -.2018652E-02, &
            -.7019151E+01, 0.3539959E-07, 0.1184749E-02, 0.3669560E-04, &
            0.1571506E+01,                                              &
            -.2217953E-03, 0.7627699E+00, -.4194946E-05, -.2065405E-02, &
            -.6071417E+01, 0.3702088E-07, 0.1469947E-02, 0.3767924E-04, &
            0.1569556E+01,                                              &
            -.2079468E-03, 0.7663283E+00, -.4394308E-05, -.2152590E-02, &
            -.6715378E+01, 0.3976722E-07, 0.1725891E-02, 0.3939064E-04, &
            0.1575804E+01,                                              &
            -.1980183E-03, 0.7689424E+00, -.4541042E-05, -.2204342E-02, &
            -.5651882E+01, 0.4149079E-07, 0.2040125E-02, 0.4049755E-04, &
            0.1576477E+01,                                              &
            -.1880196E-03, 0.7716029E+00, -.4690287E-05, -.2258003E-02, &
            -.4675918E+01, 0.4324450E-07, 0.2364191E-02, 0.4165186E-04, &
            0.1578317E+01,                                              &
            -.1779505E-03, 0.7743101E+00, -.4842054E-05, -.2313572E-02, &
            -.3785171E+01, 0.4502885E-07, 0.2698316E-02, 0.4285450E-04, &
            0.1581349E+01,                                              &
            -.1678110E-03, 0.7770643E+00, -.4996357E-05, -.2371047E-02, &
            -.2977307E+01, 0.4684440E-07, 0.3042737E-02, 0.4410644E-04, &
            0.1585634E+01,                                              &
            -.1576011E-03, 0.7798655E+00, -.5153216E-05, -.2430429E-02, &
            -.2249968E+01, 0.4869178E-07, 0.3397697E-02, 0.4540871E-04, &
            0.1591199E+01,                                              &
            -.1473205E-03, 0.7827141E+00, -.5312650E-05, -.2491719E-02, &
            -.1600778E+01, 0.5057166E-07, 0.3763447E-02, 0.4676235E-04, &
            0.1598070E+01,                                              &
            -.1369693E-03, 0.7856101E+00, -.5474682E-05, -.2554916E-02, &
            -.3596997E+00, 0.5248476E-07, 0.4140258E-02, 0.4816848E-04, &
            0.1606323E+01,                                              &
            -.1266118E-03, 0.7884817E+00, -.5638715E-05, -.2619246E-02, &
            -.6281909E-01, 0.5442865E-07, 0.4524521E-02, 0.4960657E-04, &
            0.1619364E+01,                                              &
            -.1345139E-03, 0.7873170E+00, -.5556369E-05, -.2542441E-02, &
            0.2088214E+00, 0.5179135E-07, 0.4637561E-02, 0.4800856E-04, &
            0.1586649E+01,                                              &
            -.1440706E-03, 0.7856140E+00, -.5453250E-05, -.2455005E-02, &
            0.3633611E+00, 0.4893018E-07, 0.4638929E-02, 0.4611176E-04, &
            0.1562499E+01,                                              &
            -.1665572E-03, 0.7812173E+00, -.5195383E-05, -.2280789E-02, &
            0.4399669E+00, 0.4328231E-07, 0.4324148E-02, 0.4215037E-04, &
            0.1503813E+01,                                              &
            -.1637978E-03, 0.7820706E+00, -.5240208E-05, -.2298197E-02, &
            0.4585649E+00, 0.4377662E-07, 0.4419058E-02, 0.4253318E-04, &
            0.1506559E+01,                                              &
            -.1619274E-03, 0.7826505E+00, -.5270674E-05, -.2310060E-02, &
            0.4707457E+00, 0.4411273E-07, 0.4483783E-02, 0.4279484E-04, &
            0.1508487E+01,                                              &
            -.1968036E-03, 0.7685022E+00, -.5065275E-05, -.2026572E-02, &
            0.1412806E+00, 0.3819011E-07, 0.4124199E-02, 0.3651997E-04, &
            0.2509732E+01,                                              &
            -.1959515E-03, 0.7687880E+00, -.5080013E-05, -.2032075E-02, &
            0.1407425E+00, 0.3834810E-07, 0.4150580E-02, 0.3664608E-04, &
            0.2510006E+01,                                              &
            -.1952771E-03, 0.7690145E+00, -.5091690E-05, -.2036438E-02, &
            0.1403134E+00, 0.3847329E-07, 0.4171510E-02, 0.3674620E-04, &
            0.2510234E+01,                                              &
            -.1947245E-03, 0.7692003E+00, -.5101268E-05, -.2040019E-02, &
            0.1399597E+00, 0.3857600E-07, 0.4188699E-02, 0.3682845E-04, &
            0.2510430E+01,                                              &
            -.1942468E-03, 0.7693610E+00, -.5109556E-05, -.2043119E-02, &
            0.1396524E+00, 0.3866488E-07, 0.4203585E-02, 0.3689972E-04, &
            0.2510606E+01,                                              &
            -.1938159E-03, 0.7695060E+00, -.5117036E-05, -.2045917E-02, &
            0.1393740E+00, 0.3874511E-07, 0.4217032E-02, 0.3696412E-04, &
            0.2510769E+01,                                              &
            -.1934224E-03, 0.7696385E+00, -.5123871E-05, -.2048475E-02, &
            0.1391188E+00, 0.3881842E-07, 0.4229328E-02, 0.3702303E-04, &
            0.2510922E+01,                                              &
            -.1930477E-03, 0.7697648E+00, -.5130385E-05, -.2050914E-02, &
            0.1388749E+00, 0.3888830E-07, 0.4241053E-02, 0.3707923E-04, &
            0.2511071E+01                                               /))

    ! The above coefficients were calculated using program "denscoef"
    ! (employing the UNESCO equation of state) and are valid for the following
    ! depths and T and S ranges
    !     k      depth        tmin      tmax           smin      smax
    !    24      10.00       -2.000    30.000        10.5000   41.0000
    !    23      30.00       -2.000    30.000        10.5000   41.0000
    !    22      51.00       -2.000    30.000        10.5000   41.0000
    !    21      74.00       -2.000    30.000        10.5000   41.0000
    !    20      99.50       -2.000    30.000        10.5000   41.0000
    !    19     129.00       -2.000    30.000        10.5000   41.0000
    !    18     165.00       -2.000    30.000        10.5000   41.0000
    !    17     210.50       -2.000    30.000        10.5000   41.0000
    !    16     271.00       -2.000    19.000        33.7000   36.6000
    !    15     357.00       -2.000    19.000        33.7000   36.6000
    !    14     484.00       -2.000    19.000        33.7000   36.6000
    !    13     675.00       -2.000    14.000        34.0000   35.8000
    !    12     952.50       -2.000    11.000        34.1000   35.7000
    !    11    1307.50       -1.000     7.000        34.4000   35.1000
    !    10    1700.00       -1.000     7.000        34.5000   35.1000
    !     9    2100.00       -1.000     7.000        34.6000   35.0000
    !     8    2500.00       -1.000     7.000        34.6000   35.0000
    !     7    2900.00       -1.000     7.000        34.6000   35.0000
    !     6    3300.00       -1.000     7.000        34.6000   35.0000
    !     5    3700.00       -1.000     7.000        34.6000   35.0000
    !     4    4100.00       -1.000     7.000        34.6000   35.0000
    !     3    4500.00       -1.000     7.000        34.6000   35.0000
    !     2    4900.00        0.000     7.000        34.6000   35.0000
    !     1    5300.00        0.000     7.000        34.6000   35.0000
    !
    !  The 3rd order polynomial will return density departures [gm/cm**3] as a
    ! function of potential temperature [deg C] & salinity [model units]

    REAL :: tq, sq


    ! Use a 3rd order polynomail approx to the UNESCO eqn of state.  See
    ! Bryan and Cox, 1972.  The coeffs for our levels were generated from a
    ! utility program provided with the MOM2 code.

    ! Convert s to MOM units and temperature to degrees C and subtract base
    ! values.
    
    sq = (s - 35.0) / 1000.0 - so(iz)
    tq = t - to(iz)

    rho = sigmat_ref + 1000 * &
         ((c(1, iz) + (c(4, iz) + c(7, iz) * sq) * sq + &
          (c(3, iz) + c(8, iz) * sq + c(6, iz) * tq) * tq) * tq + &
          (c(2, iz) + (c(5, iz) + c(9, iz) * sq) * sq) * sq)

  END FUNCTION eos


  !--------------------------------------------------------------------
  !
  !  Determine whether a given point is within a given ocean region.
  !

  FUNCTION point_is_in(region, ix, iy) RESULT(is_it)

    IMPLICIT NONE

    INTEGER, INTENT(IN) :: region
    INTEGER, INTENT(IN) :: ix
    INTEGER, INTENT(IN) :: iy

    LOGICAL             :: is_it


    LOGICAL, SAVE                                  :: first = .TRUE.
    INTEGER, SAVE, DIMENSION(FOAM_OLON, FOAM_OLAT) :: regions

    INTEGER :: nc_id


    ! Easy case first....

    IF (region == REGION_GLOBAL) THEN
       is_it = .TRUE.
       RETURN
    END IF


    ! Read the regions file the first time through.

    IF (first) THEN
       IF (TRIM(regions_file) == '') &
            CALL fatal(ERR_CFG_MISSING, 'foam_regions')
       nc_id = open_aux_file(regions_file)
       CALL read_2d_netcdf_var(nc_id, 'region', regions)
       first = .FALSE.
    END IF

    is_it = (regions(ix, iy) == region)

  END FUNCTION point_is_in


  !--------------------------------------------------------------------
  !
  !  This subprogram determines the mixed layer depth from provided
  !  density, salinity, and temperature profiles for a user specified
  !  temperature difference.
  !
  !  Adapted from code written by Peter A. Rochford, Naval Research
  !  Laboratory, July 2001.
  !

  SUBROUTINE mld_from_density(tdata, sdata, ddata, kmax, depth, mld)

    IMPLICIT NONE

    REAL, DIMENSION(:),           INTENT(IN)  :: tdata
    REAL, DIMENSION(SIZE(tdata)), INTENT(IN)  :: sdata
    REAL, DIMENSION(SIZE(tdata)), INTENT(IN)  :: ddata
    REAL, DIMENSION(SIZE(tdata)), INTENT(IN)  :: depth
    INTEGER,                      INTENT(IN)  :: kmax
    REAL,                         INTENT(OUT) :: mld


    REAL :: rho0, rho1
    REAL :: deltad, fieldval

    REAL, PARAMETER :: deltat = 0.8


    ! Determine density change

    rho0 = eos(tdata(FOAM_ODPTH),          sdata(FOAM_ODPTH), FOAM_ODPTH)
    rho1 = eos(tdata(FOAM_ODPTH) - deltat, sdata(FOAM_ODPTH), FOAM_ODPTH)
    deltad = ABS(rho1 - rho0)


    ! Find the Mixed Layer Depth

    fieldval = ddata(FOAM_ODPTH) + deltad
    CALL interph(fieldval, ddata(FOAM_ODPTH), deltad, ddata, &
                 depth, FOAM_ODPTH, kmax, 0.1, mld)

  END SUBROUTINE mld_from_density

  
  !--------------------------------------------------------------------
  !
  ! Subprogram to obtain the depth at which a field has the value of 
  ! fieldval = fieldref +/- fieldinc.
  !-----------------------------------------------------------------
  !
  ! 1) Arguments
  !
  !    fieldref - reference field value (see 2 and 3)
  !    fieldtol - absolute change in field value from fieldref at which 
  !               depth desired (see 2)
  !    field    - array of the field values
  !    zdepth   - array of the corresponding depths for the field values
  !    kstart   - starting level for search of field values
  !    toler    - tolerance scale factor. (See 3)
  !    depth    - depth at which the field is fieldval (output)
  !
  ! 2) The field value searched for is any occurence of the field
  !    that changes by FIELDINC from the reference field value of
  !    FIELDREF, that is any value which satisfies
  !
  !        FIELDVAL = FIELDREF +/- FIELDINC
  !
  !    As an example, consider a search for a temperature value that
  !    is 1.0 C less than a sea surface temperature of 20 C. This is
  !    a typical criteria used in defining a mixed layer depth. This 
  !    algorithm will return the depth at which a supplied temperature
  !    profile has the value of either 21.0 C or 19.0 C, whichever
  !    occurs first in the search through the temperature profile
  !    starting from point number KSTART. The depth of FIELDVAL is 
  !    determined through linear interpolation between the profile points. 
  !    The final search value used is returned as FIELDVAL.
  !
  ! 3) The algorithm updates the value of FIELDREF if it encounters a
  !    region where two or more adjacent field profile points have 
  !    changed by less than a tolerance value. This field tolerance is 
  !    set as a fraction of the input field increment value. That is by
  !    the relation
  !
  !       field tolerance = TOLER * FIELDINC
  !
  !    To obtain sensible results the user is advised to use 
  !    0.01 < toler < 1.0.
  !
  ! 4) Author:
  !    Peter A. Rochford, Sverdrup Technology Inc., May 1998
  !
  !-----------------------------------------------------------------

  SUBROUTINE interph(fieldval, fieldref, fieldinc, field, &
       zdepth, kstart, kmax, toler, depth)

    IMPLICIT NONE

    INTEGER,                      INTENT(IN)  :: kstart
    INTEGER,                      INTENT(IN)  :: kmax
    REAL, DIMENSION(:),           INTENT(IN)  :: field
    REAL, DIMENSION(SIZE(field)), INTENT(IN)  :: zdepth
    REAL,                         INTENT(IN)  :: fieldref
    REAL,                         INTENT(IN)  :: fieldinc
    REAL,                         INTENT(IN)  :: toler
    REAL,                         INTENT(IN)  :: fieldval
    REAL,                         INTENT(OUT) :: depth

    REAL, PARAMETER :: depthinit = -5.0

    INTEGER :: ifound, k, kmld, kprofmax, kprofmin
    LOGICAL :: lfound, lmld, lnewsearchval
    REAL :: delta, depmin, depmax, fldtoler, fldmin, fldmax, &
         sign, valref, valsearch
    INTEGER, DIMENSION(1) :: locs


    IF (kstart == kmax) THEN
       depth = zdepth(kmax)
       RETURN
    END IF

    ! Find depth of global minimum and maximum of profile

    locs = MINLOC(field(kmax:kstart)) ; kprofmin = locs(1) + kmax - 1
    locs = MAXLOC(field(kmax:kstart)) ; kprofmax = locs(1) + kmax - 1

    ! Find bottom level where first uniform region occurs, if it occurs
    ! at all.

    valref = fieldref
    fldtoler = toler * fieldinc
    lmld = .FALSE.
    kmld = 0
    DO k = kstart, kmax, -1
       delta = field(k - 1) - field(k)
       IF (ABS(delta) <= ABS(fldtoler) .AND. .NOT. lmld) THEN
          kmld = k - 1
          lmld = .FALSE.
       ELSE
          IF (kmld /= 0) lmld = .TRUE.
       END IF
    END DO

    ! Search for mixed layer depth (MLD)
    !
    ! 1) On first pass, search for field value taking into 
    !    account the presence of an uppermost uniform region.
    ! 2) On second pass, search for fixed change from field
    !    value at depth(kstart).

    depth = depthinit
    depmin = 0.0 ; depmax = 0.0
    fldmin = 0.0 ; fldmax = 0.0
    lfound = .FALSE. ; lnewsearchval = .TRUE.
    ifound = 1
    find_mld: DO WHILE (.NOT. lfound .AND. ifound < 3)
       ! Locate the two depths on each side of the field value

       level_loop: DO k = kstart - 1, kmax, -1
          IF (field(k + 1) < field(k)) THEN
             ! Increasing with depth
             fldmin = field(k + 1)
             fldmax = field(k)
             depmin = zdepth(k + 1)
             depmax = zdepth(k)
             sign = 1.0
          ELSE
             ! Decreasing with depth
             fldmin = field(k)
             fldmax = field(k + 1)
             depmin = zdepth(k)
             depmax = zdepth(k + 1)
             sign = -1.0
          END IF

          IF (fldmin <= valref + sign * fieldinc) THEN
             valsearch = valref + sign * fieldinc
          ELSE IF (fldmax >= valref - sign * fieldinc) THEN
             valsearch = valref - sign * fieldinc
          ELSE IF (lnewsearchval) THEN
             valsearch = valref + sign * fieldinc
             lnewsearchval = .FALSE.
          END IF
          
          ! Determine if current pair of levels contains the field value.
          field_interval: IF (valsearch < fldmax .AND. &
                              valsearch >= fldmin) THEN
             ! First check if field is uniform below this depth
             delta = field(k - 1) - field(k)
             IF (ABS(delta) <= ABS(fldtoler) .AND. k >= kmld) THEN
                ! Update field value for which the depth is desired.
                valref = field(k)
                lnewsearchval = .TRUE.
             ELSE
                ! Determine the mixed layer depth
                depth = (((depmax - depmin) / (fldmax-fldmin)) * &
                     (valsearch - fldmin)) + depmin
                lfound = .TRUE.
                EXIT
             END IF
          END IF field_interval
       END DO level_loop

       IF (depth == depthinit) THEN
          ! Repeat search using value of field(kstart)
          valref = field(kstart)
          IF (ifound == 2) depth = zdepth(kmax)
       ELSE
          IF (depth > zdepth(kprofmax) .AND. fieldinc > 0.0) THEN
             valref = field(kstart)
             lfound = .FALSE.
          ELSE IF (depth < zdepth(kprofmin) .AND. fieldinc < 0.0) THEN
             valref = field(kstart)
             lfound = .FALSE.
          END IF
       END IF
       ifound = ifound + 1
    END DO find_mld

  END SUBROUTINE interph

END MODULE ocean_utils

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
