!----------------------------------------------------------------------
!
!  File:   vars_3d.f90
!  Author: Ian Ross, University of Bristol
!  Date:   19 Feb 2005
!
!----------------------------------------------------------------------
!
!  Routines for fetching and calculating 3-D output from FOAM/LPJ.
!
!----------------------------------------------------------------------

MODULE vars_3d

  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils

  USE foam_defs                 ! FOAM-specific definitions.
  USE file_utils
  USE math_utils
  USE ocean_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: get_one_slice_3d

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 3-D data.
  !

  SUBROUTINE get_one_slice_3d(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :, :),     INTENT(OUT) :: data


    INTEGER :: nc_id

    IF (var%domain == DOMAIN_ATMOS) THEN
       CALL get_one_slice_atmos_3d(var, sday, eday, axes, data)
    ELSE
       SELECT CASE (var%name)

          ! The ocean meridional streamfunction needs to be calculated.

       CASE ('stfmmc')
          CALL get_meridional_stream_func(var, sday, eday, axes, data)

          ! This is handled as a number of 2-D variables in FOAM; they
          ! need to be collated here.

       CASE ('tsoil')
          CALL get_soil_temp(var, sday, eday, axes, data)

          ! Others are all simple time-dependent 3-D fields.

       CASE DEFAULT
          nc_id = open_history_file(foam_var_name(var%name), eday)
          CALL read_3d_netcdf_var(nc_id, foam_var_name(var%name), data)
          CALL close_file(nc_id)

       END SELECT
    END IF
  END SUBROUTINE get_one_slice_3d


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for atmospheric 3-D data.
  !

  SUBROUTINE get_one_slice_atmos_3d(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :, :),     INTENT(OUT) :: data


    INTEGER :: nc_id

    ! Temporary data.
    REAL, DIMENSION(FOAM_ALON, FOAM_ALAT, FOAM_AHGT) :: tmp

    ! Hybrid sigma vertical coordinate information.
    REAL, DIMENSION(FOAM_AHGT) :: hya, hyb
    REAL :: p0

    ! Surface pressure (for determining model pressure surfaces).
    REAL, DIMENSION(FOAM_ALON, FOAM_ALAT) :: ps

    ! Model pressure levels.
    REAL, DIMENSION(FOAM_AHGT) :: pmodel

    ! Loop counters.
    INTEGER :: lonidx, latidx


    ! Read the appropriate data field (on hybrid sigma coordinate
    ! surfaces), along with the ancillary variables needed to work out the
    ! model pressure levels.

    nc_id = open_history_file(foam_var_name(var%name), eday)
    CALL read_3d_netcdf_var(nc_id, foam_var_name(var%name), tmp)
    CALL read_1d_netcdf_var(nc_id, 'hyam', hya)
    CALL read_1d_netcdf_var(nc_id, 'hybm', hyb)
    CALL read_0d_netcdf_var(nc_id, 'P0', p0)
    CALL read_2d_netcdf_var(nc_id, 'ps', ps)
    CALL close_file(nc_id)


    ! Do one point at a time, because the model pressure levels vary from
    ! place to place as the surface pressure varies.

    DO latidx = 1, FOAM_ALAT
       DO lonidx = 1, FOAM_ALON

          ! Build the model pressure surfaces.

          pmodel = hya * p0 + hyb * ps(lonidx, latidx)


          ! Interpolate each point onto the required pressure surfaces.

          CALL press_interp(data(lonidx, latidx, :), tmp(lonidx, latidx, :), &
                            pmodel, axes(3)%values)
       END DO
    END DO


    ! The relative humidity calculation in FOAM is a tiny bit
    ! flakey....

    IF (TRIM(var%name) == 'hur') &
       WHERE (data /= FOAM_MISSING_VAL .AND. data > 100.0) data = 100.0

  END SUBROUTINE get_one_slice_atmos_3d


  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for meridional streamfunction.
  !

  SUBROUTINE get_meridional_stream_func(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :, :),     INTENT(OUT) :: data


    ! Temporary data.
    REAL, DIMENSION(FOAM_OLON, FOAM_OLAT, FOAM_ODPTH) :: vo
    INTEGER :: nc_id


    ! Read the ocean meridional velocity field.

    nc_id = open_history_file('vo', eday)
    CALL read_3d_netcdf_var(nc_id, 'vo', vo)
    CALL close_file(nc_id)


    ! Calculate the streamfunction.

    CALL calc_stfmmc(vo, data)

  END SUBROUTINE get_meridional_stream_func


  !--------------------------------------------------------------------
  !
  !  Get soil temperature.
  !

  SUBROUTINE get_soil_temp(var, sday, eday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday, eday
    TYPE(axis_def), DIMENSION(3), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :, :),     INTENT(OUT) :: data


    INTEGER :: nc_id


    ! Read the separate FOAM soil layer temperature variables.

    nc_id = open_history_file('tsoil1', eday)
    CALL read_2d_netcdf_var(nc_id, 'tsoil1', data(:, :, 1))
    CALL read_2d_netcdf_var(nc_id, 'tsoil2', data(:, :, 2))
    CALL read_2d_netcdf_var(nc_id, 'tsoil3', data(:, :, 3))
    CALL read_2d_netcdf_var(nc_id, 'tsoil4', data(:, :, 4))
    CALL close_file(nc_id)

  END SUBROUTINE get_soil_temp

END MODULE vars_3d

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
