!----------------------------------------------------------------------
!
!  File:   file_utils.f90
!  Author: Rupert Gladstone, University of Bristol
!  Date:   12 Mar 2005
!
!----------------------------------------------------------------------
!
! utilities for accessing netcdf files in UM specific format
!
!----------------------------------------------------------------------

MODULE file_utils

  USE ukmo_defs
  USE netcdf
  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils
!
!  USE foam_defs                 ! FOAM-specific definitions.
!  USE file_utils
!  USE math_utils
!  USE ocean_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: build_um_filename
  PUBLIC :: build_filenames
  PUBLIC :: open_history_file   
  PUBLIC :: close_file 
  PUBLIC :: normalise_dir
  PUBLIC :: get_um_info
  PUBLIC :: read_2d_netcdf_var
  PUBLIC :: read_2d_netcdf_subvar
  PUBLIC :: read_3d_netcdf_var
  PUBLIC :: read_3d_netcdf_subvar

CONTAINS


  !--------------------------------------------------------------------
  ! 
  !  Open the appropriate UM history file.
  ! 
    
  FUNCTION open_history_file(fname,subDir) RESULT(nc_id)
    
    IMPLICIT NONE
    
    CHARACTER, INTENT(IN) :: fname*20,subdir*4 ! file name and sub directory
                                          ! within um_data_dir (global).
    INTEGER               :: nc_id        ! netCDF ID for file.
    CHARACTER(LEN=1024)   :: filepath     ! File name.
    INTEGER               :: status       ! netCDF file open status.

    filepath = TRIM(ukmo_data_dir) // TRIM(ukmo_job) // '/' // &
         TRIM(subdir) // '/' // fname
    PRINT *, 'Opening UM output file: ',  TRIM(filepath)
    status = NF90_OPEN(filepath, NF90_NOWRITE, nc_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_OPEN, filepath)
    
  END FUNCTION open_history_file


  !--------------------------------------------------------------------
  !
  !  Close an open UM history file.
  !
  
  SUBROUTINE close_file(nc_id)
    
    IMPLICIT NONE
    
    INTEGER, INTENT(IN) :: nc_id  ! netCDF ID for file.
         
    INTEGER :: status           ! netCDF file close status.

    status = NF90_CLOSE(nc_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_CLOSE)
  
  END SUBROUTINE close_file

  !--------------------------------------------------------------------
  ! 
  ! Build UM output netcdf filenames for the given job, output stream
  ! and time slice.
  ! **************************************************************
  ! *** currently only builds filenames for first and last day ***
  ! *** of specified interval, and checks they are the same.   ***
  ! *** They are expected to be the same since the expected    ***
  ! *** interval is 30 days and all UM output files containing ***
  ! *** MOTIF variables are created at this frequency.         ***
  ! **************************************************************

  SUBROUTINE build_filenames(outstr,sub_model,sday,eday,fname)

    IMPLICIT NONE

    CHARACTER,     INTENT(IN)  :: outstr*2,sub_model
    INTEGER,       INTENT(IN)  :: sday , eday
    CHARACTER(20)              :: fname,fname_end
    INTEGER                    :: nc_id,centadj,yearadj,year,dayofyr,&
                                  monthnum
    CHARACTER                  :: century,yearstring*3
    CHARACTER, DIMENSION(12)   :: month*3
    
    fname    =build_um_filename(outstr,sub_model,sday)
    fname_end=build_um_filename(outstr,sub_model,eday)

    if (fname /= fname_end) CALL fatal(ERR_UNSUPPORTED, 'time slice too large')

  END SUBROUTINE build_filenames


  !--------------------------------------------------------------------
  !
  ! Build a UM output netcdf filename for the given job, output stream
  ! and day of run.
  ! The hard-coded timing information (century, yearadj etc) is specific
  ! to the PMIP runs.
  !

  FUNCTION build_um_filename(outstr,sub_model,sday) RESULT(fname)

    IMPLICIT NONE

    CHARACTER,     INTENT(IN)  :: outstr*2,sub_model
    INTEGER,       INTENT(IN)  :: sday
    CHARACTER(20)              :: fname
    INTEGER                    :: nc_id,centadj,yearadj,year,dayofyr,&
                                  monthnum
    CHARACTER                  :: century,yearstring*3
    CHARACTER, DIMENSION(12)   :: month*3
    
    DATA month /'jan','feb','mar','apr','may','jun','jul','aug',&
                'sep','oct','nov','dec'/

    ! work out year and month from start of main run based on sday
    year     = (sday-1)/360       ! day 360 or 1 gives year 0 or 0,
    dayofyr  = MOD((sday-1), 360) ! dayofyr 359 or 0,
    monthnum = dayofyr/30 + 1     ! and monthnum 12 or 1.

    ! Convert to absolute time in UM output file format
    ! Note: main run started at o30jan
    century = 'o'
    yearadj = 30
    year = year + yearadj             ! after adjusting year to find
    if (year/100 >= 1)  century = 'p' ! century indicator,
    year = MOD(year, 100)             ! remove centuries from year
    WRITE (yearstring, '(I3)') 100+year

    ! construct filename
    fname = TRIM(ukmo_job)//sub_model//'.'//outstr//century//yearstring(2:3)&
            //month(monthnum)//'.nc'

  END FUNCTION build_um_filename




  !--------------------------------------------------------------------
  !
  !  Normalise a directory name so that it has a slash as its last
  !  character.
  !
    
  SUBROUTINE normalise_dir(dir)
         
    IMPLICIT NONE
    
    CHARACTER(LEN=*), INTENT(INOUT) :: dir
    
    INTEGER :: last
         
    last = LEN_TRIM(dir)
    IF (dir(last:last) /= '/') dir = TRIM(dir) // '/'
  
  END SUBROUTINE normalise_dir


  !--------------------------------------------------------------------
  !
  ! Another big ugly lookup table.  Looking up fiddly bits needed in
  ! filename construction.
  ! note: valid var domains are O(cean),S(urface),A(tmos),I(ce)
  
  SUBROUTINE get_um_info(var,um_name,out_str,sub_model)
        
    IMPLICIT NONE

    TYPE(variable_def),           INTENT(IN)  :: var


    CHARACTER (LEN=30),INTENT(OUT) :: um_name
    CHARACTER (LEN=2), INTENT(OUT) :: out_str
    CHARACTER,         INTENT(OUT) :: sub_model


    SELECT CASE (var%name) 

!    CASE ('ts')
!        SELECT CASE (var%domain)
!
!        CASE('')  !OSAI) 
!            um_name='temp_mm_srf'
!            out_str='pd' 
!            sub_model='a'
!    
!            um_name='temp_mm_srf'
!            out_str='pd'
!            sub_model='o'
!
!        END SELECT


    CASE ('tas')
        um_name='temp_mm_1_5m'
        out_str='pd'
        sub_model='a'

    CASE ('hurs') 
        um_name='rh_mm_1_5m' 
        out_str='pd' 
        sub_model='a' 

    CASE ('sos') 
        um_name='salinity_mm_dpth'         
        out_str='pf'  
        sub_model='o'         

    CASE ('sozm')         
        um_name='salinity_mm_dpth'         
        out_str='pf'         
        sub_model='o'         

    CASE ('thetaozm')
        um_name='temp_mm_dpth'
        out_str='pf'
        sub_model='o'

    CASE ('uos')         
        um_name='ucurrTot_mm_dpth'         
        out_str='pf'         
        sub_model='o'         

    CASE ('vos')
        um_name='vcurrTot_mm_dpth'
        out_str='pf'
        sub_model='o'


    CASE ('cheight')
        um_name='canopyHeight_mm_srf'
        out_str='pi'
        sub_model='a'
        
    CASE ('cl')         
        um_name='cldamount_mm_hyb'         
        out_str='pd'         
        sub_model='a'         
 
    CASE ('clivi') 
        um_name='QCF_mm_hyb' 
        out_str='pd' 
        sub_model='a' 

    CASE ('clt')         
        um_name='totCloudranmax_mm_ua'         
        out_str='pd'         
        sub_model='a'         

    CASE ('clwvi')
        um_name='QCL_mm_hyb'
        out_str='pd'
        sub_model='a'

    CASE ('esoil')         
        um_name='soilEvap_mm_srf'         
        out_str='pi'         
        sub_model='a'         
         
    CASE ('evspsblveg')
        um_name='canopyEvap_mm_can'
        out_str='pi'
        sub_model='a'

    CASE ('gpp')         
        um_name='GPP_mm_surface'         
        out_str='pi'         
        sub_model='a'         
         
    CASE ('gpppft')
        um_name='GPP_PFT_mm_surface'
        out_str='pi'
        sub_model='a'

    CASE ('hcont300')         
        um_name='temp_mm_dpth'         
        out_str='pf'         
        sub_model='o'         

    CASE ('hfls')
        um_name='lh_mm_srf'
        out_str='pd'
        sub_model='a'

    CASE ('hfss')         
        um_name='sensHflx_T_mm_srf'         
        out_str='pi'         
        sub_model='a'         
         
    CASE ('hresp')
        um_name='soilResp_mm_srf'
        out_str='pi'
        sub_model='a'

    CASE ('hur')         
        um_name='rh_mm_p'         
        out_str='pc'         
        sub_model='a'         

    CASE ('huss')
        um_name='q_mm_1_5m'
        out_str='pd'  
        sub_model='a'

    CASE ('intuaw')         
        um_name='uq_mm_p'         
        out_str='pc'         
        sub_model='a'         

    CASE ('intvaw')
        um_name='vq_mm_p'
        out_str='pc'
        sub_model='a'

    CASE ('laipft')         
        um_name='LAI_PFT_mm_srf'         
        out_str='pi'         
        sub_model='a'         

    CASE ('lcarcont')
        um_name='GBMlitCarb_snp_srf'
        out_str='pi'
        sub_model='a'

    CASE ('lcarcontpft')         
        um_name='LitCarbPFT_snp_srf'         
        out_str='pi'         
        sub_model='a'         

    CASE ('mrros')
        um_name='srfRunoff_mm_srf'
        out_str='pd'
        sub_model='a'

    CASE ('mrso')         
        um_name='sm_mm_srf'         
        out_str='pd'         
        sub_model='a'         

    CASE ('npp')
        um_name='NPP_mm_srf'
        out_str='pi'  
        sub_model='a'

    CASE ('npppft')         
        um_name='NPP_PFT_mm_srf'         
        out_str='pi'         
        sub_model='a'         
         
    CASE ('pftfrac')
        um_name='fracPFTs_mm_srf'
        out_str='pi'
        sub_model='a'

    CASE ('potevap')         
        um_name='potEvap_mm_srf'         
        out_str='pi'         
        sub_model='a'         
         
    CASE ('potevappft')
        um_name='potEvapTiles_mm_srf'
        out_str='pi'
        sub_model='a'

    CASE ('prc')         
        um_name='cvrain_mm_srf'         
        out_str='pd'         
        sub_model='a'         

    CASE ('pr')
        um_name='precip_mm_srf'
        out_str='pd' 
        sub_model='a'

    CASE ('prw')
        um_name='QCL_mm_srf'
        out_str='pd'
        sub_model='a'

    CASE ('ps')         
        um_name='p_mm_srf'         
        out_str='pd'         
        sub_model='a'         
         
    CASE ('psl')
        um_name='p_mm_srf'
        out_str='pd'
        sub_model='a'

    CASE ('rlds')         
        um_name='ilr_mm_s3_srf'         
        out_str=''         
        sub_model=''         
         
    CASE ('rldscs')
        um_name='csilr_mm_s3_srf'
        out_str='pd'  
        sub_model='a'

    CASE ('rls')         
        um_name='longwave_mm_s3_srf'         
        out_str='pd'         
        sub_model='a'         
         
    CASE ('rlut')
        um_name='olr_mm_s3_TOA'
        out_str='pd'
        sub_model='a'

    CASE ('rlutcs')         
        um_name='csolr_mm_s3_TOA'         
        out_str='pd'         
        sub_model='a'         
         
    CASE ('rsds')
        um_name='downSol_Seaice_mm_s3_srf'
        out_str='pd'
        sub_model='a'

    CASE ('rsdscs')         
        um_name='clrskyDownSol_mm_s3_srf'         
        out_str='pd'         
        sub_model='a'         
         

    CASE ('rsdt') 
        um_name='downSol_mm_TOA'
        out_str='pd'
        sub_model='a'

    CASE ('rss')         
        um_name='solar_mm_s3_srf'         
        out_str='pd'         
        sub_model='a'         
         
    CASE ('rsuscs')         
        um_name='clrskyUpSol_mm_s3_srf'         
        out_str='pd'         
        sub_model='a'         


    CASE ('rsut')
        um_name='upSol_mm_s3_TOA'
        out_str='pd'
        sub_model='a'
 
    CASE ('rsutcs')         
        um_name='clrskyUpSol_mm_s3_TOA'         
        out_str='pd'         
        sub_model='a'         

    CASE ('scarcont')
        um_name='soilCarbon_mm_srf'
        out_str='pi'
        sub_model='a'

    CASE ('siarea')         
        um_name='iceconc_mm_uo'         
        out_str='pf'         
        sub_model='o'         

    CASE ('sic')
        um_name='iceconc_mm_uo'
        out_str='pf'
        sub_model='o'

    CASE ('siext')         
        um_name='iceconc_mm_uo'         
        out_str='pf'         
        sub_model='o'         

    CASE ('sildarea')
        um_name='iceconc_mm_uo'   
        out_str='pf'
        sub_model='o'

    CASE ('sisnvol')         
        um_name='snowdepthonseaice_mm_uo'         
        out_str='pf'         
        sub_model='o'         
         
    CASE ('sit')
        um_name='icedepth_mm_uo'
        out_str='pf'
        sub_model='o'

    CASE ('snc')         
        um_name='snowCover_mm_srf'         
        out_str='pi'         
        sub_model='o'         

    CASE ('snd')
        um_name='snowdepth_mm_srf'
        out_str='pi'
        sub_model='o'
 
    CASE ('snm')         
        um_name='snowmelt_mm_srf'         
        out_str='pd'         
        sub_model='o'         

    CASE ('so')
        um_name='salinity_mm_dpth'
        out_str='pf' 
        sub_model='o'

    CASE ('stfbarot')         
        um_name='streamFn_mm_uo'         
        out_str='pf'         
        sub_model='o'         
         
    CASE ('tasmax')
        um_name='temp_dma_1_5m'
        out_str='pb'  
        sub_model='a'

    CASE ('tasmin')         
        um_name='temp_dmi_1_5m'         
        out_str='pb'         
        sub_model='a'         

    CASE ('thetao')
        um_name='temp_mm_dpth'
        out_str='pf'  
        sub_model='o'

    CASE ('tos')         
        um_name='temp_mm_uo'         
        out_str='pf'         
        sub_model='o'         

    CASE ('trans')
        um_name='transpiration_mm_srf'
        out_str='pd'  
        sub_model='a'

    CASE ('tsoil')         
        um_name='soiltemp_mm_soil'         
        out_str='pd'         
        sub_model='a'         
         
    CASE ('uas')
        um_name='u_mm_10m'  
        out_str='pd'  
        sub_model='a'

    CASE ('uo')         
        um_name='ucurrTot_mm_dpth'         
        out_str='pf'         
        sub_model='o'         
         
    CASE ('usi')
        um_name='uVelSeaice_mm_uo'   
        out_str='pf'
        sub_model='o'

    CASE ('vas')         
        um_name='v_mm_10m'         
        out_str='pd'         
        sub_model='a'         

    CASE ('vcarcont')         
        um_name='GBMvegCarb_snp_srf'         
        out_str='pi'         
        sub_model='a'         

    CASE ('vcarcontpft') 
        um_name='VegCarbPFT_snp_srf'   
        out_str='pi'   
        sub_model='a' 

    CASE ('vo')
        um_name='vcurrTot_mm_dpth'
        out_str='pf'
        sub_model='o'

    CASE ('vsi')         
        um_name='vVelSeaice_mm_uo'         
        out_str='pf'         
        sub_model='o'         

    CASE ('wo')
        um_name='W_mm_dpth'
        out_str='pf'
        sub_model='o'

    CASE ('ziso20')         
        um_name='temp_mm_dpth'         
        out_str='pf'         
        sub_model='o'         

    CASE ('zmlo')
        um_name='mixLyrDpth_mm_uo'  
        out_str='pf'  
        sub_model='o'
    
    CASE DEFAULT
       CALL fatal(ERR_MODEL_INVALID_VAR, var%name)
    END SELECT
  
  END SUBROUTINE get_um_info


  !--------------------------------------------------------------------
  !
  !  Read a 3-D netCDF variable: assumes that the output data parameter is
  !  the right size! 
  !     
    
  SUBROUTINE read_3d_netcdf_var(nc_id, var, data)
        
    IMPLICIT NONE
        
    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.

    REAL, DIMENSION(:,:,:),      INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id

    print *, 'read_3d_netcdf_var: var=', TRIM(var), '  data=', &
         SIZE(data,1), 'x', SIZE(data,2), 'x', SIZE(data,3)
    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &   
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))
        
    status = NF90_GET_VAR(nc_id, var_id, data, &
         count = (/ SIZE(data, 1), SIZE(data, 2), SIZE(data, 3), 1 /))
    IF (status /= NF90_NOERR) &   
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))
        
  END SUBROUTINE read_3d_netcdf_var

  
  !-----------------------------------------------------------------------
  !
  !  Read a 2-D netCDF variable: assumes that the output data parameter is
  !  the right size!
  !

  SUBROUTINE read_2d_netcdf_var(nc_id, um_name, data)

    IMPLICIT NONE

    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=30),           INTENT(IN)  :: um_name! Variable name.

    REAL, DIMENSION(:,:),        INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id

    status = NF90_INQ_VARID(nc_id, um_name, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(um_name))

    status = NF90_GET_VAR(nc_id, var_id, data)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(um_name))

  END SUBROUTINE read_2d_netcdf_var


  !--------------------------------------------------------------------
  !
  !  Read a 3-D slice from a 4-D netCDF variable: assumes that the
  !  output data parameter is the right size!
  !
  
  SUBROUTINE read_3d_netcdf_subvar(nc_id, var, data, day)
  
    IMPLICIT NONE
  
    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.
    INTEGER,                     INTENT(IN)  :: day

    REAL, DIMENSION(:,:,:),      INTENT(OUT) :: data   ! Output data.
  
  
    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id
  
    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))
    
    status = NF90_GET_VAR(nc_id, var_id, data, &
         start = (/ 1, 1, 1, day /), &
         count = (/ SIZE(data, 1), SIZE(data, 2), SIZE(data, 3), 1 /))
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))
                                          
  END SUBROUTINE read_3d_netcdf_subvar


  !--------------------------------------------------------------------
  !
  !  Read a 2-D slice from a 4-D netCDF variable: assumes that the
  !  output data parameter is the right size!
  !

  SUBROUTINE read_2d_netcdf_subvar(nc_id, var, data, day)

    IMPLICIT NONE

    INTEGER,                     INTENT(IN)  :: nc_id  ! netCDF file ID.
    CHARACTER(LEN=VAR_NAME_LEN), INTENT(IN)  :: var    ! Variable name.
    INTEGER,                     INTENT(IN)  :: day

    REAL, DIMENSION(:,:),        INTENT(OUT) :: data   ! Output data.


    INTEGER :: status                        ! netCDF status.
    INTEGER :: var_id

    status = NF90_INQ_VARID(nc_id, var, var_id)
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to access netCDF variable: ' // TRIM(var))

    status = NF90_GET_VAR(nc_id, var_id, data, &
         start = (/ 1, 1, 1, day /), &
         count = (/ SIZE(data, 1), SIZE(data, 2), 1, 1 /))
    IF (status /= NF90_NOERR) &
         CALL fatal(ERR_MODEL_FILE_READ, &
         'Failed to read netCDF variable: ' // TRIM(var))

  END SUBROUTINE read_2d_netcdf_subvar

END MODULE file_utils
