!----------------------------------------------------------------------
!
!  File:   proc_utils.f90
!  Author: Rupert Gladstone, University of Bristol
!  Date:   12 Mar 2005
!
!----------------------------------------------------------------------
!
! utilities for processing data from netcdf files 
!
!----------------------------------------------------------------------

MODULE proc_utils

  USE ukmo_defs
  USE netcdf
  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils
  USE file_utils
!  USE math_utils
!  USE ocean_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: lat_lon_bounds
  PUBLIC :: get_sub_dir
  PUBLIC :: interp_to_press

CONTAINS


  !--------------------------------------------------------------------
  !
  
  SUBROUTINE interp_to_press(data_tmp, data, axes,sday)

    IMPLICIT NONE

    REAL, DIMENSION(:,:,:),     INTENT(IN) :: data_tmp
    TYPE(axis_def),DIMENSION(3),INTENT(IN) :: axes
    INTEGER,                    INTENT(IN) :: sday

    REAL, DIMENSION(:,:,:),ALLOCATABLE,INTENT(OUT):: data

    REAL, DIMENSION(:,:),ALLOCATABLE   :: ps
    REAL, DIMENSION(19)                :: A,B, pmodel
    INTEGER                            :: lat,lon,nc_id
    CHARACTER (LEN=30)                 :: fname

    ALLOCATE(data(SIZE(axes(1)%values),SIZE(axes(2)%values),&
        SIZE(axes(3)%values)))
    ALLOCATE(ps(SIZE(axes(1)%values),SIZE(axes(2)%values)))

    CALL get_eta_coeffs(A,B)

    ! open file and extract surface pressure
    fname = build_um_filename('pd', 'a', sday)
    nc_id = open_history_file(fname, 'pcpd')
    CALL read_2d_netcdf_var(nc_id, 'p_mm_srf', ps)
    CALL close_file(nc_id)

    ! loop over horiz grid, interpolating pressure at each point
    DO lat = 1, SIZE(data_tmp,2)
        DO lon = 1, SIZE(data_tmp,1)
            pmodel = A + B * ps(lon,lat)
            CALL press_interp(data(lon, lat, :), data_tmp(lon, lat, :), &
                            pmodel, axes(3)%values)
        END DO
    END DO

  END SUBROUTINE interp_to_press        

  !--------------------------------------------------------------------
  !  
  ! Get the A and B components of the eta hybrid pressure coord.
  ! Eta values are hard coded here for the standard BRIDGE UM setup.
  !
  ! eta       =  A/P0 + B
  ! where A/P0 and B are pressure referenced to P0 and Pstar 
  ! respectively (B is sigma)
  !
  ! We cannot get pressure from eta alone, we need to decompose it 
  ! into A and B, then calculate pressure as follows:
  ! pressure  =  A + B * Pstar

  SUBROUTINE get_eta_coeffs(A,B)

    REAL, DIMENSION(19),INTENT(OUT) :: A,B         ! the hybrid coord coeffs

    REAL, DIMENSION(19)             :: eta         ! the hybrid coord
    INTEGER                         :: levels,i    ! the number of vertical levels
    REAL                            :: eta_s,eta_p ! see below
    REAL                            :: P0

    P0 = 100000.

    eta_s = 0.75       ! eta on the half level above the top pure sigma level
    eta_p = 0.04       ! eta on the half level below the bottom pure p/p0 level
                       ! (currently set for levels 5 and 17)
    levels = SIZE(eta) !

    ! These are the eta*1000 values from a standard BRIDGE UM run
    eta = (/996.998901, 974.955933, 930.416748, 869.832275, 792.228455, 699.574402, &
             599.503235, 504.521851, 422.103119, 354.697723, 299.751617, 249.701767, &
             199.626816, 149.501251, 99.246773, 56.854244, 29.594332, 14.797166, 4.605880 /)
    eta = eta / 1000.

    ! set the A and B values
    DO i = 1,LEVELS
      IF (eta(i) > eta_s) THEN
        ! The lower 5 levels are sigma only
        A(i) = 0
      ELSE IF (eta(i) < eta_p) THEN
        ! The levels from 17 up are pressure only
        A(i) = eta(i)
      ELSE 
        ! this formula is taken from the UM code
        A(i) = (eta_s-eta(i))**2 * (eta(i)*(eta_s+eta_p) - 2.0*eta_p**2) /(eta_s-eta_p)**3
      END IF
    END DO
    B = eta - A
    A = A * P0

  END SUBROUTINE

! extract of UM code:
!C Calculate hybrid coordinate parameter A                                  ABCALC1.177   
!      Z1=(ETA_S-ETA_P)*(ETA_S-ETA_P)*(ETA_S-ETA_P)                         ABCALC1.178   
!      DO JL=1,LEVELS+1                                                     ABCALC1.179   
!        IF(ETAH(JL).GE.ETA_S) THEN                                         ABCALC1.180   
!          AH(JL)=0.0                                                       ABCALC1.181   
!        ELSE IF(ETAH(JL).LE.ETA_P) THEN                                    ABCALC1.182   
!          AH(JL)=ETAH(JL)                                                  ABCALC1.183   
!        ELSE                                                               ABCALC1.184   
!          Z2=ETAH(JL)*(ETA_S+ETA_P)-2.0*ETA_P*ETA_P                        ABCALC1.185   
!          AH(JL)=(ETA_S-ETAH(JL))*(ETA_S-ETAH(JL))*Z2/Z1                   ABCALC1.186   
!          IF(ABS(AH(JL)).LT.TINY) AH(JL)=0.0                               ABCALC1.187   
!        END IF                                                             ABCALC1.188   
!      END DO                                        

  !--------------------------------------------------------------------
  !
  !  Get the sub directory corresponding to the given output stream
  !

  FUNCTION get_sub_dir(out_str) RESULT(sub_dir)

    IMPLICIT NONE

    CHARACTER, INTENT(IN) :: out_str*2
    CHARACTER             :: sub_dir*4 

    SELECT CASE (out_str)

    CASE ('pc','pd')
        sub_dir = 'pcpd'
    
    CASE DEFAULT
        sub_dir = out_str
    
    END SELECT

  END FUNCTION get_sub_dir

     
  !--------------------------------------------------------------------
  !
  !  Calculate bounds for latitude and longitude grids for UM.
  !
        
  SUBROUTINE lat_lon_bounds(axis, values, bounds)
        
    IMPLICIT NONE
        
    CHARACTER(LEN=AXIS_NAME_LEN),       INTENT(IN)  :: axis
    REAL, DIMENSION(:),                 INTENT(IN)  :: values

    REAL, DIMENSION(:, :), ALLOCATABLE, INTENT(OUT) :: bounds

    INTEGER :: alloc_stat
    REAL :: cellsize

    
    ! Note: cellsize can be negative for descending axis values
    cellsize = values(2)-values(1) ! assumes cellsize is regular
        
    
    SELECT CASE (axis)
        
        ! Longitude axes: both the atmosphere and the ocean use
        ! regular longitude axes
            
    CASE (AXIS_LON)
        ALLOCATE(bounds(2, size(values)), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        bounds(1, :) = values - cellsize / 2.0
        bounds(2, :) = values + cellsize / 2.0
        
        ! ensure bounds cyclic
!        where (bounds(:,:)<0.)
!            bounds(:,:)=360.+bounds(:,:)
!        else where (bounds(:,:)>360.)
!            bounds(:,:) = bounds(:,:)-360.
!        end where
    
       
       ! Latitude axes: both the atmosphere and the ocean use
       ! regular latitude axes
    
    CASE (AXIS_LAT)
        ALLOCATE(bounds(2, size(values)), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        bounds(1, :) = values - cellsize / 2.0
        bounds(2, :) = values + cellsize / 2.0
        
        
        ! ensure bounds stop at poles
        where (bounds(:,:)<-90.)
            bounds(:,:)=-90.
        else where (bounds(:,:)>90.)
            bounds(:,:) = 90.
        end where
        
    END SELECT

  END SUBROUTINE lat_lon_bounds


  !--------------------------------------------------------------------
  !
  !  Interpolation between pressure levels using log interpolation.
  !
          
  SUBROUTINE press_interp(out, in, pin, pout)
             
    IMPLICIT NONE
          
    REAL, DIMENSION(:),         INTENT(OUT) :: out   ! Output data.
             
    REAL, DIMENSION(:),         INTENT(IN)  :: in    ! Input data.
    REAL, DIMENSION(SIZE(in)),  INTENT(IN)  :: pin   ! Input pressure levels.
    REAL, DIMENSION(SIZE(out)), INTENT(IN)  :: pout  ! Output pressure levels.
       
          
    INTEGER :: outlev, icheck
    INTEGER :: below
    LOGICAL :: exact
    REAL :: po, p1, p2
    REAL :: in1, in2
       
    DO outlev = 1, SIZE(out)
       po = pout(outlev)
       
       ! Find bracketing input pressures.

       exact = .FALSE.
       below = 0
       DO icheck = 1, SIZE(in) - 1
          IF (pin(icheck) == po) THEN
             below = icheck
             exact = .TRUE.
             EXIT
          END IF
          IF (pin(icheck) > po .AND. pin(icheck + 1) < po) THEN
             below = icheck
             EXIT
          END IF
       END DO
       IF (pin(SIZE(in)) == po) THEN
          below = SIZE(in)
          exact = .TRUE.
       END IF
       
       ! Do interpolation: values of output pressure outside of the input
       ! pressure range produce missing values (they may, for instance,
       ! correspond to instances where the pressure level reaches the
       ! Earth's surface).

       IF (below == 0) THEN
          out(outlev) = UKMO_MISSING_VAL 
       ELSE IF (exact) THEN
          out(outlev) = in(below)
       ELSE
          p1 = pin(below)     ; in1 = in(below)
          p2 = pin(below + 1) ; in2 = in(below + 1)
          out(outlev) = in1 + LOG(p1 / po) * (in2 - in1) / LOG(p1 / p2)
       END IF
    END DO

  END SUBROUTINE press_interp

END MODULE proc_utils

