!----------------------------------------------------------------------
!
!  File:   vars_2d.f90
!  Author: Some Jokers
!  Date:   12 Mar 2005
!
!----------------------------------------------------------------------
!
!  Routines for fetching and calculating 2-D output from UKMO.
!
!----------------------------------------------------------------------

MODULE vars_2d

  USE pp_defs                   ! Main post-processor definitions.
  USE pp_utils

  USE ukmo_defs                 ! FOAM specific definitions.
  USE file_utils
  USE proc_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: get_daily_2d_slice
  PUBLIC :: get_monthly_2d_slice

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Get interval averaged value for 2-D data.
  !

  SUBROUTINE get_monthly_2d_slice(var, sday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data

    !use temporary data storage for procesing some variables
    REAL, DIMENSION(:,:,:), ALLOCATABLE       :: data_tmp
    CHARACTER(LEN=30)       :: um_name
    CHARACTER(LEN=2)        :: out_str
    CHARACTER               :: sub_model
    CHARACTER(LEN=20)       :: fname
    CHARACTER(LEN=4)        :: sub_dir
    INTEGER                 :: nc_id,n
    


    data(:,:)=0

    SELECT CASE (var%name)

    ! surface albedo calculated from total down and net down SW rad.
    CASE ('albs')

        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 2))
        um_name='downSol_Seaice_mm_s3_srf' ! miss-named, tot down sw all srf
        out_str='pd'
        sub_model='a'      
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        print*,'um_name: ',um_name
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)

        um_name='solar_mm_s3_srf' ! net down sw surface
        out_str='pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,2)) 
        CALL close_file(nc_id)  
    
        data = UKMO_MISSING_VAL
        ! albedo = tot sw up / tot sw down.  
        ! leave at missing val for regions of 24hr darkness.
        WHERE (data_tmp(:,:,1)/=0)
            data(:,:) = (data_tmp(:,:,1) - data_tmp(:,:,2))/data_tmp(:,:,1)
        END WHERE


    ! (net down clr-sky LW) = (tot down clrsky LW) - (tot up clrsky LW)
    ! where (tot up clrsky LW) = (tot down LW) - (net down LW) 
    CASE ('rlscs')

        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 3))
        um_name=  'csilr_mm_s3_srf' ! clrsky tot down LW
        out_str=  'pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)

        um_name=    'ilr_mm_s3_srf' ! tot down LW
        out_str=    'pd'
        sub_model=  'a'
        sub_dir = get_sub_dir(out_str)  
        fname = build_um_filename(out_str, sub_model, sday)  
        nc_id = open_history_file(fname, sub_dir)  
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,2))  
        CALL close_file(nc_id)  
     
        um_name='longwave_mm_s3_srf' !net down LW
        out_str='pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,3))
        CALL close_file(nc_id)

        data(:,:) = data_tmp(:,:,1)-data_tmp(:,:,2)+data_tmp(:,:,3)


    ! calc net down clrsky SW from tot down and tot up.
    CASE ('rsscs')
        
        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 2))
        um_name=  'clrskyDownSol_mm_s3_srf' ! clrsky tot down SW
        out_str=  'pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday) 
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)
        
        um_name=    'clrskyUpSol_mm_s3_srf' ! clrsky tot up SW
        out_str=    'pd'
        sub_model=  'a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,2))
        CALL close_file(nc_id)
            
        data(:,:) = data_tmp(:,:,1)-data_tmp(:,:,2)
        

    ! net down clrsky toa sw.  Calc from down minus up. 
    CASE ('rstcs')
        
        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 2))
        um_name=  'clrskyUpSol_mm_s3_TOA' ! clrsky tot up SW TOA
        out_str=  'pd'
        sub_model='a'   
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)
        
        um_name=    'downSol_mm_TOA' ! tot down SW TOA
        out_str=    'pd'
        sub_model=  'a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,2))
        CALL close_file(nc_id)
        
        data(:,:) = data_tmp(:,:,2)-data_tmp(:,:,1)                


    ! calc up LW rad at surf from tot down and net down
    CASE ('rlus')
    
        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 2))
        um_name=  'ilr_mm_s3_srf' ! tot down lw srf
        out_str=  'pd'
        sub_model='a' 
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)
        
        um_name=    'longwave_mm_s3_srf' ! net down LW srf
        out_str=    'pd'
        sub_model=  'a' 
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,2))
        CALL close_file(nc_id)
        
        data(:,:) = data_tmp(:,:,1)-data_tmp(:,:,2)


    ! toa net down SW calc from tot up and down
    CASE ('rst')
        
        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 2))
        um_name=  'downSol_mm_TOA' ! tot down sw toa
        out_str=  'pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)
        
        um_name=    'upSol_mm_s3_TOA' ! tot up sw toa
        out_str=    'pd'
        sub_model=  'a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,2))
        CALL close_file(nc_id)
        
        data(:,:) = data_tmp(:,:,1)-data_tmp(:,:,2)
        


    ! tot srf up sw is tot down minus net down
    CASE ('rsus')
    
        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 2))
        um_name=  'downSol_Seaice_mm_s3_srf' ! miss-named, tot down sw srf
        out_str=  'pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)
        
        um_name=    'solar_mm_s3_srf' ! net down sw srf
        out_str=    'pd'
        sub_model=  'a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,2))
        CALL close_file(nc_id)
        
        data(:,:) = data_tmp(:,:,1)-data_tmp(:,:,2)


    ! net down sw+lw at toa
    ! calc as tot_down_sw - tot_up_sw - tot_up_lw        
    CASE ('rtmt')
    
        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 3))
        um_name=  'downSol_mm_TOA' ! tot down sw toa
        out_str=  'pd'
        sub_model='a' 
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)
        
        um_name=    'olr_mm_s3_TOA' ! tot up lw toa
        out_str=    'pd'
        sub_model=  'a' 
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,2))
        CALL close_file(nc_id)

        um_name=    'upSol_mm_s3_TOA' ! tot up sw toa
        out_str=    'pd'
        sub_model=  'a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,3))
        CALL close_file(nc_id)

        data(:,:) = data_tmp(:,:,1)-data_tmp(:,:,2)-data_tmp(:,:,3)



!        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), ))
!        um_name=
!        out_str=
!        sub_model=        
!        sub_dir = get_sub_dir(out_str)
!        fname = build_um_filename(out_str, sub_model, sday)
!        nc_id = open_history_file(fname, sub_dir)
!        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,))
!        CALL close_file(nc_id)

    ! net downward heat = SW rad + LW rad + latent + sensible (all net down)
!
    CASE ('hfns')

        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 5))
        um_name='lh_mm_srf'  ! latent heat surface, +ve upwards
        out_str='pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,1))
        CALL close_file(nc_id)

        um_name=  'sensHflx_T_mm_srf' ! sensible heat on tiles, +ve upwards
        out_str=  'pi'
        sub_model=   'a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        CALL get_vert_mean(fname,sub_dir,um_name,data_tmp(:,:,2),axes,8)

        um_name='sh_mm_srf'  ! sensible heat ocean surface, +ve upwards
        out_str='pd'               
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,3))
        CALL close_file(nc_id)
        
        um_name=  'solar_mm_s3_srf' ! net down sw surface
        out_str=  'pd'
        sub_model=   'a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,4)) 
        CALL close_file(nc_id)
        
        um_name='longwave_mm_s3_srf' !net down LW surface
        out_str='pd'
        sub_model='a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data_tmp(:,:,5))
        CALL close_file(nc_id)
        
        ! set missing val to zero (we are combining land, ocean, and everywhere variables)
        WHERE (data_tmp == UKMO_MISSING_VAL)
            data_tmp = 0
        END WHERE

        data = - data_tmp(:,:,1) - data_tmp(:,:,2) - &
            data_tmp(:,:,3) + data_tmp(:,:,4) + data_tmp(:,:,5)


    ! Triffid data is obtained at 10 day intervals.  
    ! Need to average over month
    CASE ('lcarcont','vcarcont')

        ALLOCATE(data_tmp(SIZE(axes(1)%values), SIZE(axes(2)%values), 1))
        CALL get_um_info(var, um_name, out_str, sub_model)
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        DO n=1,3
            CALL read_2d_netcdf_subvar(nc_id, um_name, data_tmp(:,:,1), n)
            data(:,:) = data(:,:) + data_tmp(:,:,1)
        END DO
        CALL close_file(nc_id)
        data(:,:) = data(:,:) / 3

    CASE DEFAULT
        CALL get_um_info(var, um_name, out_str, sub_model)
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_var(nc_id, um_name, data)
        CALL close_file(nc_id)

    END SELECT

  END SUBROUTINE get_monthly_2d_slice



  !--------------------------------------------------------------------
  !
  !  Get the data and calculate the mean over the vert/level/pseudo axis.
  !     
    
  SUBROUTINE get_vert_mean(fname,sub_dir,um_name,data,axes,axis_len)
  
    IMPLICIT NONE
 
    CHARACTER(LEN=20),          INTENT(IN) :: fname
    CHARACTER(LEN=4),           INTENT(IN) :: sub_dir 
    CHARACTER(LEN=30),          INTENT(IN) :: um_name
    TYPE(AXIS_DEF),DIMENSION(2),INTENT(IN) :: axes   

    REAL, DIMENSION(:,:), INTENT(OUT)      :: data

    INTEGER                                :: axis_len
    REAL, DIMENSION(:,:,:), ALLOCATABLE    :: data_tmp
    INTEGER                                :: i, nc_id

    data = 0

    ALLOCATE(data_tmp(SIZE(data,1),SIZE(data,2),axis_len))
    nc_id = open_history_file(fname, sub_dir)
    CALL read_3d_netcdf_var(nc_id, um_name, data_tmp)
    CALL close_file(nc_id)

    data = data_tmp(:,:,1)

    ! sum values
    DO i = 2,axis_len
        WHERE (data /= UKMO_MISSING_VAL)
            data = data + data_tmp(:,:,i)
        END WHERE 
    END DO

    ! take mean
    WHERE (data /= UKMO_MISSING_VAL)
        data = data / axis_len
    END WHERE

  END SUBROUTINE get_vert_mean



  !--------------------------------------------------------------------
  !
  !  Get daily value for 2-D data.
  !

  SUBROUTINE get_daily_2d_slice(var, sday, axes, data)

    IMPLICIT NONE

    ! Variable, day to get and axes.
    TYPE(variable_def),           INTENT(IN)  :: var
    INTEGER,                      INTENT(IN)  :: sday
    TYPE(axis_def), DIMENSION(2), INTENT(IN)  :: axes

    ! Output data.
    REAL, DIMENSION(:, :),        INTENT(OUT) :: data


    CHARACTER(LEN=30) :: um_name
    CHARACTER(LEN=2)  :: out_str
    CHARACTER         :: sub_model
    CHARACTER(LEN=20) :: fname
    CHARACTER(LEN=4)  :: sub_dir
    INTEGER :: nc_id
    

    SELECT CASE (var%name)

    CASE ('tas')
        um_name = 'temp_dm_1_5m'
        out_str = 'pb'
        sub_model = 'a'
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_subvar(nc_id, um_name, data, MOD(sday - 1, 30) + 1)
        CALL close_file(nc_id)

    CASE DEFAULT
        CALL get_um_info(var, um_name, out_str, sub_model)
        sub_dir = get_sub_dir(out_str)
        fname = build_um_filename(out_str, sub_model, sday)
        nc_id = open_history_file(fname, sub_dir)
        CALL read_2d_netcdf_subvar(nc_id, um_name, data, MOD(sday - 1, 30) + 1)
        CALL close_file(nc_id)

    END SELECT

  END SUBROUTINE get_daily_2d_slice

END MODULE vars_2d

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
