!----------------------------------------------------------------------
!
!  File:   pp_motif.f90
!  Author: Ian Ross, University of Bristol
!  Date:   17 Feb 2005
!
!----------------------------------------------------------------------
!
!  Main program for MOTIF/PMIP2 file post-processor.
!
!----------------------------------------------------------------------
!
!  COMMAND LINE ARGUMENTS
!
!  Four command line arguments are required:
!
!   1. The path to the configuration file containing the fixed
!      configuration information (such as experiment IDs,
!      organisation name and so on).
!   2. The MOTIF variable name to be processed.
!   3. The CMOR table name to which the variable refers.
!   4. The model output times to be processed, as a day range,
!      counting from the beginning of the model run (for instance,
!      361-36361 processes 100 years of data, skipping the first year,
!      assuming a 360 day year).
!
!----------------------------------------------------------------------
!
!  CONFIGURATION FILE
!
!  The first command line argument is the path to a configuration
!  file, each line of which should be of the form: <key>:<value>. 
!  The valid keys come from two sets, one common to all models, and
!  one model-specific.
!
!----------------------------------------------------------------------

PROGRAM pp_motif

  USE cmor_users_functions

  USE pp_defs
  USE pp_utils
  USE pp_vars

  USE model_main

  IMPLICIT NONE


  ! Configuration file entry array and count.

  TYPE(cfg_arg), DIMENSION(MAX_CFG_ARGS) :: cfg_args
  INTEGER                                :: cfg_entries


  ! Command line arguments.

  CHARACTER(LEN=ARG_LEN) :: cfg_path       ! Config. file path.
  CHARACTER(LEN=ARG_LEN) :: var_name       ! Variable name.
  CHARACTER(LEN=ARG_LEN) :: table_name     ! Table name.
  CHARACTER              :: table_domain   ! Table domain (e.g. atmos.).
  CHARACTER(LEN=2)       :: table_freq     ! Table output frequency.
  CHARACTER(LEN=ARG_LEN) :: output_times   ! Output time range.
  INTEGER                :: s_day, e_day   ! Output start and end days.


  ! Mandatory configuration file entries.

  CHARACTER(LEN=ARG_LEN) :: table_path   ! Path to table files.
  CHARACTER(LEN=ARG_LEN) :: expt_id      ! Experiment ID.
  CHARACTER(LEN=ARG_LEN) :: institution  ! Responsible institution.


  ! Optional configuration file entries.

  CHARACTER(LEN=ARG_LEN) :: cfg_val_tmp        ! Temporary value.
  CHARACTER(LEN=ARG_LEN) :: output_path = '.'  ! Output path.
  CHARACTER(LEN=ARG_LEN) :: contact_name = ''  ! Researcher contact name.


  ! Variable axis information.

  INTEGER                            :: axis_count      ! Count of axes.
  INTEGER                            :: s_axis_count    ! Singleton axes.
  TYPE(axis_def), DIMENSION(4)       :: axes            ! Axis info.
  INTEGER, DIMENSION(:), ALLOCATABLE :: cmor_axis_ids   ! CMOR axis IDs.
  INTEGER                            :: tmp_axis_id     ! Special for singles.


  ! Flags for the existence of different kinds of special axes.

  LOGICAL :: has_time_axis                ! Time axis?


  ! Variable information.

  INTEGER                :: var_id        ! CMOR variable ID.
  TYPE(variable_def)     :: var           ! Collated variable info.
  CHARACTER(LEN=ARG_LEN) :: positive_dir  ! Positive direction for fluxes.


  ! Model specific information.

  CHARACTER(LEN=ARG_LEN)       :: description  ! Model description.
  TYPE(calendar_info)          :: cal_info     ! Model calendar info.
  CHARACTER(LEN=UNIT_NAME_LEN) :: model_units  ! Model's units for variable.
  REAL                         :: missing_val  ! Model's missing value.


  ! Time handling variables.

  LOGICAL :: fixed_steps                       ! Fixed/variable timestepping?
  INTEGER :: step                              ! Step len. for fixed stepping.
  INTEGER :: year                              ! Total year length.
  INTEGER, DIMENSION(:), ALLOCATABLE :: steps  ! Steps for variable stepping.
  INTEGER :: start_step                        ! First step for var. stepping.
  INTEGER :: start_day, end_day                ! Request time range.
  INTEGER :: true_start, true_end              ! True time range.
  INTEGER :: istep                             ! Counter for var. stepping.


  ! Data storage: note that each of the 'normal' arrays holds one time-slice
  ! of data; the time axis is handled separately.  There are also seasonal
  ! data arrays, with an extra dimension (for the month) used to collect
  ! monthly data over the course of the run for final seasonal averaging.

  REAL                                  :: data_0d  ! Singleton data.
  REAL, DIMENSION(:), ALLOCATABLE       :: data_1d  ! 1-D data.
  REAL, DIMENSION(:,:), ALLOCATABLE     :: data_2d  ! 2-D data.
  REAL, DIMENSION(:,:,:), ALLOCATABLE   :: data_3d  ! 3-D data.
  REAL, DIMENSION(:), ALLOCATABLE       :: data_0ds ! Singleton data.
  REAL, DIMENSION(:,:), ALLOCATABLE     :: data_1ds ! 1-D data.
  REAL, DIMENSION(:,:,:), ALLOCATABLE   :: data_2ds ! 2-D data.
  REAL, DIMENSION(:,:,:,:), ALLOCATABLE :: data_3ds ! 3-D data.
  INTEGER :: alloc_stat


  ! Information for seasonal averaging.

  INTEGER, DIMENSION(12) :: seasonal_count
  INTEGER                :: month


  ! Temporaries and loop counters.

  DOUBLE PRECISION, DIMENSION(1)   :: times        ! CMOR time index.
  DOUBLE PRECISION, DIMENSION(2,1) :: time_bounds  ! CMOR time bounds.
  INTEGER                          :: iaxis        ! Axis counter.
  INTEGER                          :: flag         ! CMOR return value.
  INTEGER                          :: ix, iy, iz   ! Averaging counters.


  !--------------------------------------------------------------------
  !
  !  MAIN PROGRAM STARTS HERE
  !
  !--------------------------------------------------------------------

  ! Process the command line and configuration file.

  CALL process_command_line(cfg_path, var_name, table_name, output_times)
  CALL process_config_file(cfg_path, cfg_args, cfg_entries)


  ! Determine domain and frequency information from table name (where
  ! possible).  This assumes that the table names are of the form
  ! PMIP2_table_<domain>_<freq> (with <domain> a single character, and
  ! <freq> a two character string.

  table_domain = table_name(13:13)
  table_freq = table_name(15:16)


  ! Extract mandatory information from the configuration file.

  table_path  = get_cfg_value(cfg_args, cfg_entries, 'table_path')
  expt_id     = get_cfg_value(cfg_args, cfg_entries, 'expt_id')
  institution = get_cfg_value(cfg_args, cfg_entries, 'institution')
  IF (TRIM(table_path) == '')     CALL fatal(ERR_CFG_MISSING, 'table_path')
  IF (TRIM(expt_id) == '')        CALL fatal(ERR_CFG_MISSING, 'expt_id')
  IF (TRIM(institution) == '')    CALL fatal(ERR_CFG_MISSING, 'institution')


  ! Extract optional information from the configuration file.

  cfg_val_tmp = get_cfg_value(cfg_args, cfg_entries, 'output_path')
  IF (TRIM(cfg_val_tmp) /= '') output_path = cfg_val_tmp
  cfg_val_tmp = get_cfg_value(cfg_args, cfg_entries, 'contact')
  IF (TRIM(cfg_val_tmp) /= '') contact_name = cfg_val_tmp


  ! Initialise the model-specific interface, allowing the model code to
  ! pick up any configuration information it needs.

  CALL model_init(cfg_args, cfg_entries, description, cal_info, missing_val)
  year = SUM(cal_info%mon_lens)


  ! Process output time range command line argument.

  CALL process_time_range(output_times, s_day, e_day)
  CALL analyse_timestepping(s_day, e_day, table_freq, cal_info, &
                            fixed_steps, step, steps, start_step, &
                            true_start, true_end)
  IF (true_start == 0 .OR. true_end == 0) CALL fatal(ERR_EMPTY_TIME_RANGE)


  ! Initialise CMOR.

  IF (cmor_setup(table_path, 'replace', set_verbosity=0) /= 0) &
       CALL fatal(ERR_CMOR_SETUP)


  ! Initialise CMOR dataset.

  IF (cmor_dataset(output_path, expt_id, institution, source=description, &
                   calendar=cal_info%name, realization=1, &
                   contact=contact_name, &
                   history='Written by MOTIF post-processor') /= 0) &
     CALL fatal(ERR_CMOR_DATASET)


  ! Get variable axis information.

  CALL setup_axes(var_name, table_domain, table_freq, axes, axis_count)


  ! Count singleton axes.  These should always occur at the end of the axis
  ! list.

  s_axis_count = 0
  DO iaxis = 1, axis_count
     IF (TRIM(axes(iaxis)%name) == AXIS_HGT1 .OR. &
         TRIM(axes(iaxis)%name) == AXIS_HGT2 .OR. &
         TRIM(axes(iaxis)%name) == AXIS_Pm   .OR. &
         TRIM(axes(iaxis)%name) == AXIS_Pb) THEN
        s_axis_count = s_axis_count + 1
     ELSE
        IF (s_axis_count > 0) CALL fatal(ERR_SINGLETON_AXIS)
     END IF
  END DO


  ! Set up CMOR axes.  The time axis and geographic region axes are
  ! treated differently to normal spatial axes.  Singleton pressure axes
  ! also need special treatment.

  has_time_axis = .FALSE.

  ALLOCATE(cmor_axis_ids(axis_count - s_axis_count), STAT=alloc_stat)
  IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)

  DO iaxis = 1, axis_count - s_axis_count
     SELECT CASE (TRIM(axes(iaxis)%name))
     CASE (AXIS_TIME)
        cmor_axis_ids(iaxis) = cmor_axis(table_name, axes(iaxis)%name, &
             units=axes(iaxis)%units)
        has_time_axis = .TRUE.

     CASE (AXIS_REGION)
        cmor_axis_ids(iaxis) = cmor_axis(table_name, axes(iaxis)%name, &
             units='none', length=4, coord_vals=region_names)

     CASE (AXIS_HEMI)
        cmor_axis_ids(iaxis) = cmor_axis(table_name, axes(iaxis)%name, &
             units='none', length=2, coord_vals=hemi_names)

     CASE (AXIS_LON, AXIS_LAT)
        cmor_axis_ids(iaxis) = cmor_axis(table_name, axes(iaxis)%name, &
             units=axes(iaxis)%units, length=SIZE(axes(iaxis)%values), &
             coord_vals=axes(iaxis)%values, cell_bounds=axes(iaxis)%bounds)

     CASE DEFAULT
        cmor_axis_ids(iaxis) = cmor_axis(table_name, axes(iaxis)%name, &
             units=axes(iaxis)%units, length=SIZE(axes(iaxis)%values), &
             coord_vals=axes(iaxis)%values)
     END SELECT
     IF (axes(iaxis)%cmor_id < 0) CALL fatal(ERR_CMOR_AXIS, axes(iaxis)%name)
  END DO


  ! Deal with singleton axes.

  DO iaxis = axis_count - s_axis_count + 1, axis_count
     tmp_axis_id = cmor_axis(table_name, axes(iaxis)%name, &
          units=axes(iaxis)%units, length=1, coord_vals=axes(iaxis)%values)
     IF (tmp_axis_id < 0) CALL fatal(ERR_CMOR_AXIS, axes(iaxis)%name)
  END DO


  ! Set up CMOR variable.

  model_units = model_get_units(var_name)
  CALL setup_positive(var_name, positive_dir)
  IF (TRIM(positive_dir) == '') THEN
     var_id = cmor_variable(table_name, var_name, units=model_units, &
          axis_ids=cmor_axis_ids, missing_value=missing_val)
  ELSE
     var_id = cmor_variable(table_name, var_name, units=model_units, &
          axis_ids=cmor_axis_ids, missing_value=missing_val, &
          positive=positive_dir)
  END IF
  IF (var_id < 0) CALL fatal(ERR_CMOR_VAR, var_name)
  var%domain = table_domain
  var%freq = table_freq
  var%name = var_name


  ! For seasonal averaging, everything's a bit different, so deal with it
  ! separately....

  IF (var%freq /= FREQ_SEASONAL) THEN
     SELECT CASE (axis_count - s_axis_count)
        ! Two axes can either be 1-D data with time-dependence, or non time-
        ! dependent 2-D data.
     CASE (2)
        IF (has_time_axis) THEN
           ALLOCATE(data_1d(SIZE(axes(1)%values)), STAT=alloc_stat)
           IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        ELSE
           ALLOCATE(data_2d(SIZE(axes(1)%values), SIZE(axes(2)%values)), &
                STAT=alloc_stat)
           IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        END IF

        ! This is always time-dependent 2-D data.
     CASE (3)
        ALLOCATE(data_2d(SIZE(axes(1)%values), SIZE(axes(2)%values)), &
             STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)

        ! This is always time-dependent 3-D data.
     CASE (4)
        ALLOCATE(data_3d(SIZE(axes(1)%values), SIZE(axes(2)%values), &
             SIZE(axes(3)%values)), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
     END SELECT


     ! Deal with the case of non-time-dependent data, which should only be
     ! 2-D land surface data.

     IF (.NOT. has_time_axis) THEN
        IF (axis_count /= 2 .OR. s_axis_count /= 0) &
             CALL fatal(ERR_NON_2D_FIXED)


        ! Get and write the data - the time values aren't important here.

        CALL model_get_value(var, 1, 1, axes(1:2), data_2d)
        times(1) = 0.0
        flag = cmor_write(var_id, data_2d, ntimes_passed=1, time_vals=times)

        IF (flag < 0) CALL fatal(ERR_CMOR_WRITE)
     ELSE
        ! Run through the required time steps, extracting data from the
        ! model-specific code and writing it out via CMOR.

        start_day = true_start
        IF (.NOT. fixed_steps) istep = start_step

        DO
           IF (fixed_steps) THEN
              end_day = start_day + step - 1
           ELSE
              end_day = start_day + steps(istep) - 1
              istep = MOD(istep, SIZE(steps)) + 1
           END IF
           IF (end_day > true_end) EXIT


           ! Get and write the appropriate shape of data.

           times(1) = end_day
           time_bounds(:,1) = (/ start_day - 1, end_day /)

           SELECT CASE (axis_count - s_axis_count)
           CASE (1)
              CALL model_get_value(var, start_day, end_day, data_0d)
              data_1d(1) = data_0d
              flag = cmor_write(var_id, data_1d, ntimes_passed=1, &
                   time_vals=times, time_bnds=time_bounds)

           CASE (2)
              CALL model_get_value(var, start_day, end_day, axes(1:1), data_1d)
              flag = cmor_write(var_id, data_1d, ntimes_passed=1, &
                   time_vals=times, time_bnds=time_bounds)

           CASE (3)
              CALL model_get_value(var, start_day, end_day, axes(1:2), data_2d)
              flag = cmor_write(var_id, data_2d, ntimes_passed=1, &
                   time_vals=times, time_bnds=time_bounds)

           CASE (4)
              CALL model_get_value(var, start_day, end_day, axes(1:3), data_3d)
              flag = cmor_write(var_id, data_3d, ntimes_passed=1, &
                   time_vals=times, time_bnds=time_bounds)
           END SELECT

           IF (flag < 0) CALL fatal(ERR_CMOR_WRITE)

           start_day = end_day + 1
        END DO
     END IF
  ELSE
     ! Seasonal data - we allocate enough space for 12 months of the relevant
     ! variable, then collect monthly data for the whole averaging period,
     ! finally producing a single 12-month seasonal cycle for the variable.

     ! Allocate storage space - basically just 12 times the normal amount of
     ! data storage.

     SELECT CASE (axis_count - s_axis_count)
     CASE (1)
        ALLOCATE(data_0ds(12), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        ALLOCATE(data_1ds(1, 12), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        data_0ds = 0.0

     CASE (2)
        ALLOCATE(data_1ds(SIZE(axes(1)%values), 12), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        ALLOCATE(data_1d(SIZE(axes(1)%values)), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        data_1ds = 0.0

     CASE (3)
        ALLOCATE(data_2ds(SIZE(axes(1)%values), SIZE(axes(2)%values), 12), &
             STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        ALLOCATE(data_2d(SIZE(axes(1)%values), SIZE(axes(2)%values)), &
             STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)

        data_2ds = 0.0

     CASE (4)
        ALLOCATE(data_3ds(SIZE(axes(1)%values), SIZE(axes(2)%values), &
             SIZE(axes(3)%values), 12), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        ALLOCATE(data_3d(SIZE(axes(1)%values), SIZE(axes(2)%values), &
             SIZE(axes(3)%values)), STAT=alloc_stat)
        IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
        data_3ds = 0.0
     END SELECT


     ! Run through the required time steps, extracting data from the
     ! model-specific code and storing it away for averaging.

     start_day = true_start
     istep = start_step
     seasonal_count = 0
     DO
        month = istep
        IF (fixed_steps) THEN
           end_day = start_day + step - 1
        ELSE
           end_day = start_day + steps(istep) - 1
           istep = MOD(istep, SIZE(steps)) + 1
        END IF
        IF (end_day > true_end) EXIT


        ! Get and store the appropriate shape of data.

        times(1) = end_day
        time_bounds(:,1) = (/ start_day - 1, end_day /)

        SELECT CASE (axis_count - s_axis_count)
        CASE (1)
           CALL model_get_value(var, start_day, end_day, data_0d)
           IF (data_0d == missing_val) THEN
              data_0ds(month) = missing_val
           ELSE
              data_0ds(month) = data_0ds(month) + data_0d
           END IF

        CASE (2)
           CALL model_get_value(var, start_day, end_day, &
                axes(1:1), data_1d)
           WHERE (data_1d == missing_val .OR. &
                  data_1ds(:, month) == missing_val)
              data_1ds(:, month) = missing_val
           ELSEWHERE
              data_1ds(:, month) = data_1ds(:, month) + data_1d
           END WHERE

        CASE (3)
           CALL model_get_value(var, start_day, end_day, &
                axes(1:2), data_2d)
           WHERE (data_2d == missing_val .OR. &
                  data_2ds(:, :, month) == missing_val)
              data_2ds(:, :, month) = missing_val
           ELSEWHERE
              data_2ds(:, :, month) = data_2ds(:, :, month) + data_2d
           END WHERE

        CASE (4)
           CALL model_get_value(var, start_day, end_day, &
                axes(1:3), data_3d)
           WHERE (data_3d == missing_val .OR. &
                  data_3ds(:, :, :, month) == missing_val)
              data_3ds(:, :, :, month) = missing_val
           ELSEWHERE
              data_3ds(:, :, :, month) = data_3ds(:, :, :, month) + data_3d
           END WHERE
        END SELECT
        seasonal_count(month) = seasonal_count(month) + 1
        start_day = end_day + 1
        IF (TRIM(var%name) == 'rsdt' .AND. ALL(seasonal_count == 1)) EXIT
     END DO


     ! Average the data.

     SELECT CASE (axis_count - s_axis_count)
     CASE (1)
        WHERE (seasonal_count /= 0 .AND. data_0ds /= missing_val) &
             data_0ds = data_0ds / seasonal_count

     CASE (2)
        DO ix = 1, SIZE(data_1ds, 1)
           WHERE (seasonal_count /= 0 .AND. &
                data_1ds(ix, :) /= missing_val) &
                data_1ds(ix, :) = data_1ds(ix, :) / seasonal_count
        END DO

     CASE (3)
        DO ix = 1, SIZE(data_2ds, 1)
           DO iy = 1, SIZE(data_2ds, 2)
              WHERE (seasonal_count /= 0 .AND. &
                   data_2ds(ix, iy, :) /= missing_val) &
                   data_2ds(ix, iy, :) = data_2ds(ix, iy, :) / seasonal_count
           END DO
        END DO

     CASE (4)
        DO ix = 1, SIZE(data_3ds, 1)
           DO iy = 1, SIZE(data_3ds, 2)
              DO iz = 1, SIZE(data_3ds, 3)
                 WHERE (seasonal_count /= 0 .AND. &
                      data_3ds(ix, iy, iz, :) /= missing_val) &
                      data_3ds(ix, iy, iz, :) = &
                      data_3ds(ix, iy, iz, :) / seasonal_count
              END DO
           END DO
        END DO
     END SELECT


     ! Write the data.

     start_day = 1
     DO month = 1, 12
        end_day = start_day + steps(month) - 1
        times(1) = end_day
        time_bounds(:,1) = (/ start_day - 1, end_day /)

        SELECT CASE (axis_count - s_axis_count)
        CASE (1)
           data_1ds(1, month) = data_0ds(month)
           flag = cmor_write(var_id, data_1ds(:, month), ntimes_passed=1, &
                time_vals=times, time_bnds=time_bounds)

        CASE (2)
           flag = cmor_write(var_id, data_1ds(:, month), ntimes_passed=1, &
                time_vals=times, time_bnds=time_bounds)

        CASE (3)
           flag = cmor_write(var_id, data_2ds(:, :, month), ntimes_passed=1, &
                time_vals=times, time_bnds=time_bounds)

        CASE (4)
           flag = cmor_write(var_id, data_3ds(:, :, :, month), &
                ntimes_passed=1, &
                time_vals=times, time_bnds=time_bounds)
        END SELECT

        IF (flag < 0) CALL fatal(ERR_CMOR_WRITE)
        start_day = end_day + 1
     END DO
  END IF


  ! Finalise the CMOR output file.

  IF (cmor_close() < 0) CALL fatal(ERR_CMOR_CLOSE)

CONTAINS

  !--------------------------------------------------------------------
  !
  !  INTERNAL ROUTINES START HERE
  !
  !--------------------------------------------------------------------

  !--------------------------------------------------------------------
  !
  !  Set up all axis definitions for the given variable.
  !

  SUBROUTINE setup_axes(var_name, domain, freq, axes, axis_count)

    IMPLICIT NONE

    CHARACTER(LEN=ARG_LEN), INTENT(IN)        :: var_name ! Variable name.
    CHARACTER, INTENT(IN)                     :: domain   ! Table domain.
    CHARACTER(LEN=2), INTENT(IN)              :: freq     ! Output frequency.
    TYPE(axis_def), DIMENSION(4), INTENT(OUT) :: axes     ! Axis info.
    INTEGER, INTENT(OUT)                      :: axis_count

    CHARACTER(LEN=AXIS_NAME_LEN), DIMENSION(4) :: axis_names  ! Required axes.
    INTEGER :: iaxis

    CALL get_axes(var_name, freq, axis_names, axis_count)
    axes%domain = domain
    axes(1:axis_count)%name = axis_names
    DO iaxis = 1, axis_count
       axes(iaxis)%units = motif_axis_units(axes(iaxis)%name)

       SELECT CASE (TRIM(axes(iaxis)%name))

       CASE (AXIS_P17, AXIS_P3, AXIS_Pbt, AXIS_Pbm, AXIS_Pb, AXIS_Pm)
          CALL fill_pressure_axis(axes(iaxis))

          ! The next two are both geographical region axes, so the values
          ! are really character values.  We set up some dummy values to
          ! make sizing data arrays simpler in the main program.

       CASE (AXIS_REGION)
          ALLOCATE(axes(iaxis)%values(4), STAT=alloc_stat)
          IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
          axes(iaxis)%values = (/ 1, 2, 3, 4 /)

       CASE (AXIS_HEMI)
          ALLOCATE(axes(iaxis)%values(2), STAT=alloc_stat)
          IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
          axes(iaxis)%values = (/ 1, 2 /)

          ! Do nothing here.  The time axis is handled differently.

       CASE (AXIS_TIME)

          ! All other axes should be set up by the model-specific code.

       CASE DEFAULT
          CALL model_get_axis(axes(iaxis), var_name)

       END SELECT
    END DO

  END SUBROUTINE setup_axes


  !--------------------------------------------------------------------
  !
  !  Set up positive direction marker for a given variable if necessary.
  !

  SUBROUTINE setup_positive(var_name, positive_dir)

    IMPLICIT NONE

    CHARACTER(LEN=ARG_LEN), INTENT(IN)  :: var_name      ! Variable name.
    CHARACTER(LEN=ARG_LEN), INTENT(OUT) :: positive_dir  ! Positive direction.


    positive_dir = ''

    SELECT CASE (TRIM(var_name))

    CASE ('hfls', 'hfns', 'hfss', 'rlds', 'rldscs', 'rls', 'rlscs',  &
          'rlus', 'rlut', 'rlutcs', 'rsds', 'rsdscs', 'rsdt', 'rss', &
          'rsscs', 'rst', 'rstcs', 'rsus', 'rsuscs', 'rsut', 'rsutcs', &
          'rtmt', 'hfsib')
       positive_dir = model_get_positive(var_name)
    END SELECT

  END SUBROUTINE setup_positive


  !--------------------------------------------------------------------
  !
  !  Check and process the analysis time range argument.
  !

  SUBROUTINE process_time_range(times, start, end)

    IMPLICIT NONE

    CHARACTER(LEN=ARG_LEN), INTENT(IN) :: times
    INTEGER, INTENT(OUT) :: start
    INTEGER, INTENT(OUT) :: end

    INTEGER :: dashpos

    dashpos = INDEX(times, '-')
    IF (dashpos == 0) CALL fatal(ERR_INVALID_TIME_RANGE, times)

    READ (times(:dashpos-1), 1) start
    READ (times(dashpos+1:), 1) end

1   FORMAT (I)

  END SUBROUTINE process_time_range


  !--------------------------------------------------------------------
  !
  !  Determine the time-stepping requirements based on the output
  !  frequency and the model-specific calendar.
  !

  SUBROUTINE analyse_timestepping(s_day, e_day, freq, cal_info, &
                                  fixed_steps, step, steps, start_step, &
                                  true_start, true_end)

    INTEGER, INTENT(IN)             :: s_day, e_day ! Req. start and end day.
    CHARACTER(LEN=2), INTENT(IN)    :: freq         ! Output frequency.
    TYPE(calendar_info), INTENT(IN) :: cal_info     ! Model calendar info.

    ! There are two possibilities: either we are going to do fixed size
    ! timesteps (daily or annual), or we may have timesteps of varying day
    ! length depending on the calendar we've been given.  'Seasonal' time-
    ! stepping means to use monthly timestepping, but to collect all the data
    ! over the course of the run for final seasonal averaging.
    !
    ! For fixed timestepping, fixed_steps == .TRUE. and step is used to
    ! record the size of the timestep (in days).
    !
    ! For non-fixed timestepping, fixed_steps == .FALSE. and steps is used
    ! to keep track of the different timesteps to use: the stepping code
    ! should run through the steps array, using each step size in turn,
    ! starting with the start_step'th entry.

    LOGICAL, INTENT(OUT)                              :: fixed_steps
    INTEGER, INTENT(INOUT)                            :: step
    INTEGER, DIMENSION(:), ALLOCATABLE, INTENT(INOUT) :: steps
    INTEGER, INTENT(OUT)                              :: start_step

    ! These record the real start and end days - these will not always
    ! correspond to the requested days, since we only average whole months,
    ! seasons or years.

    INTEGER, INTENT(OUT) :: true_start, true_end


    INTEGER :: year        ! Total year length.
    INTEGER :: istep       ! Loop counter.
    INTEGER :: alloc_stat


    year = SUM(cal_info%mon_lens)
    true_start = s_day
    true_end = e_day
    start_step = 1

    ! ===> THIS DOESN'T YET HANDLE HIGH-FREQUENCY OUTPUT OR LEAP YEARS. 
    ! ===> BOTH SHOULD EVENTUALLY BE DEALT WITH PROPERLY.  (LEAP YEARS CAN
    ! ===> BE DONE BY MULTIPLYING THE LENGTHS OF THE steps ARRAY BY 400
    ! ===> AND FILLING IN APPROPRIATE VALUES THROUGHOUT THE LEAP YEAR
    ! ===> CYCLE.  ANNUAL OUTPUT WILL ALSO BECOME A NON-FIXED STEP OUTPUT
    ! ===> IF DONE THIS WAY, THOUGH ONLY FOR CALENDARS WITH LEAP YEARS.)

    SELECT CASE (table_freq)

       ! We will have to support the high time-resolution output sometime,
       ! but not for the moment.

    CASE (FREQ_EXTRA_DAILY, FREQ_EXTRA_12HRLY, FREQ_EXTRA_6HRLY)
       CALL fatal(ERR_UNSUPPORTED, 'high-frequency output')

       ! The daily case is simple - fixed output frequency, and the true
       ! start and end are the same as the requested.

    CASE (FREQ_DAILY)
       fixed_steps = .TRUE.
       step = 1

       ! The annual case is also pretty simple - the only wrinkle is that
       ! the start and end dates need to be pinned to beginning and end of
       ! year days.

    CASE (FREQ_ANNUAL)
       fixed_steps = .TRUE.
       step = year
       IF (MOD(s_day - 1, year) /= 0) &
            true_start = (s_day - 1 + year - 1) / year * year + 1
       IF (MOD(e_day - 1, year) /= 0) &
            true_end = e_day / year * year

       ! A little more complicated now - monthly output and seasonal
       ! output potentially has non-uniform timesteps (think of the
       ! normal Gregorian calendar); also the start and end days need
       ! to be pinned to days at the beginning and end of whole months.

    CASE (FREQ_MONTHLY, FREQ_SEASONAL)
       fixed_steps = .FALSE.
       ALLOCATE(steps(12), STAT=alloc_stat)
       IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
       steps = cal_info%mon_lens
       IF (MOD(s_day - 1, year) /= 0) THEN
          true_start = (s_day - 1) / year * year + 1
          istep = 1
          DO WHILE (true_start + steps(istep) < s_day)
             true_start = true_start + steps(istep)
             istep = MOD(istep, 12) + 1
          END DO
          true_start = true_start + steps(istep)
          start_step = MOD(istep, 12) + 1
       END IF
       IF (MOD(e_day - 1, year) /= 0) THEN
          true_end = e_day / year * year
          istep = 1
          DO WHILE (true_end + steps(istep) <= e_day)
             true_end = true_end + steps(istep)
             istep = MOD(istep, 12) + 1
          END DO
       END IF
    END SELECT

    IF (true_start > true_end) THEN
       true_start = 0
       true_end = 0
    END IF

  END SUBROUTINE analyse_timestepping

END PROGRAM pp_motif

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
