!----------------------------------------------------------------------
!
!  File:   pp_utils.f90
!  Author: Ian Ross, University of Bristol
!  Date:   17 Feb 2005
!
!----------------------------------------------------------------------
!
!  Utility routine definitions used in MOTIF/PMIP2 file post-processor.
!
!----------------------------------------------------------------------

MODULE pp_utils

  USE pp_defs

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: process_command_line
  PUBLIC :: process_config_file
  PUBLIC :: get_cfg_value
  PUBLIC :: fatal

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Extract the command line arguments.
  !

  ! ===> NOTE THAT THIS IS PROBABLY COMPILER SPECIFIC AND SHOULD BE PULLED
  ! ===> OUT INTO A SEPARATE MODULE.

  SUBROUTINE process_command_line(cfg_path, var_name, table_name, output_times)

    IMPLICIT NONE

    CHARACTER(LEN=ARG_LEN) :: cfg_path     ! Config. file path.
    CHARACTER(LEN=ARG_LEN) :: var_name     ! Variable name.
    CHARACTER(LEN=ARG_LEN) :: table_name   ! Table name.
    CHARACTER(LEN=ARG_LEN) :: output_times ! Start and end times.


    IF (IARGC() /= 4) CALL fatal(ERR_BAD_COMMAND_LINE)

    CALL GETARG(1, cfg_path)
    CALL GETARG(2, var_name)
    CALL GETARG(3, table_name)
    CALL GETARG(4, output_times)

    RETURN

  END SUBROUTINE process_command_line


  !--------------------------------------------------------------------
  !
  !  Read and process the configuration file.
  !

  SUBROUTINE process_config_file(cfg_path, cfg_args, cfg_entries)

    IMPLICIT NONE

    ! Configuration file path.
    CHARACTER(LEN=ARG_LEN), INTENT(IN) :: cfg_path

    ! Configuration file contents (as key/value pairs).
    TYPE(cfg_arg), DIMENSION(MAX_CFG_ARGS), INTENT(OUT) :: cfg_args

    ! Number of valid entries in configuration file.
    INTEGER, INTENT(OUT) :: cfg_entries

  
    INTEGER, PARAMETER :: CFG_LUN = 11 ! LUN to use for config. file.

    LOGICAL             :: file_status  ! File existence indicator.
    INTEGER             :: nline = 0    ! Config. file line count.
    CHARACTER(LEN=2048) :: buffer       ! Line buffer
    INTEGER             :: colpos       ! Position of colon.
    INTEGER             :: icheck       ! Loop counter.


    INQUIRE (FILE=cfg_path, EXIST=file_status)
    IF (.NOT. file_status) CALL fatal(ERR_BAD_CFG_FILE, cfg_path)

    OPEN (CFG_LUN, FILE=cfg_path, ACTION='READ', STATUS='OLD')
    cfg_entries = 0
    DO
       ! Read a line.

       READ (CFG_LUN, 1, END=100) buffer
1      FORMAT (A2048)
       nline = nline + 1


       ! Skip blank lines and comment lines.

       buffer = TRIM(buffer)
       IF (TRIM(buffer) == '') CYCLE
       IF (buffer(1:1) == '!') CYCLE


       ! Extract the key and value from the line and store them.

       colpos = INDEX(buffer, ':')
       IF (colpos == 0) CALL fatal(ERR_INVALID_CFG, iarg=nline)
       cfg_entries = cfg_entries + 1
       IF (cfg_entries > MAX_CFG_ARGS) CALL fatal(ERR_TOO_MANY_CFG_ENTRIES)
       cfg_args(cfg_entries)%key = TRIM(ADJUSTL(buffer(1 : colpos - 1)))
       cfg_args(cfg_entries)%value = TRIM(ADJUSTL(buffer(colpos + 1 : )))


       ! Check for duplicate keys.

       DO icheck = 1, cfg_entries - 1
          IF (cfg_args(icheck)%key == cfg_args(cfg_entries)%key) &
               CALL fatal(ERR_DUPLICATE_CFG, iarg=nline)
       END DO
    END DO

100 CONTINUE

  END SUBROUTINE process_config_file


  !--------------------------------------------------------------------
  !
  !  Find a single configuration value given its key.
  !

  FUNCTION get_cfg_value(cfg_args, cfg_entries, key) RESULT (value)

    IMPLICIT NONE

    TYPE(cfg_arg), DIMENSION(MAX_CFG_ARGS), INTENT(IN) :: cfg_args
    INTEGER, INTENT(IN)                                :: cfg_entries
    CHARACTER(LEN=*), INTENT(IN)                       :: key

    CHARACTER(LEN=ARG_LEN)                             :: value
    INTEGER :: icheck

    value = ''
    DO icheck = 1, cfg_entries
       IF (TRIM(key) == TRIM(cfg_args(icheck)%key)) THEN
          value = TRIM(cfg_args(icheck)%value)
          EXIT
       END IF
    END DO

  END FUNCTION get_cfg_value


  !--------------------------------------------------------------------
  !
  !  Print a fatal error message and exit.
  !

  SUBROUTINE fatal(errcode, carg, iarg)

    IMPLICIT NONE

    INTEGER,                    INTENT(IN) :: errcode
    CHARACTER(LEN=*), OPTIONAL, INTENT(IN) :: carg
    INTEGER, OPTIONAL,          INTENT(IN) :: iarg


    SELECT CASE (errcode)
    CASE (ERR_BAD_COMMAND_LINE)
       PRINT *, 'Wrong number of command line arguments'

    CASE (ERR_INVALID_TIME_RANGE)
       PRINT 1, TRIM(carg)

    CASE (ERR_EMPTY_TIME_RANGE)
       PRINT *, 'Empty output time range for given output frequency'

    CASE (ERR_BAD_CFG_FILE)
       PRINT 2, TRIM(carg)

    CASE (ERR_CFG_MISSING)
       PRINT 3, TRIM(carg)

    CASE (ERR_INVALID_CFG)
       PRINT 4, iarg

    CASE (ERR_DUPLICATE_CFG)
       PRINT 5, iarg

    CASE (ERR_TOO_MANY_CFG_ENTRIES)
       PRINT *, 'Too many entries in configuration file'

    CASE (ERR_CMOR_SETUP)
       PRINT *, 'Failed to set up CMOR'
     
    CASE (ERR_CMOR_DATASET)
       PRINT *, 'Failed to set up CMOR dataset'

    CASE (ERR_CMOR_AXIS)
       PRINT 6, TRIM(carg)

    CASE (ERR_CMOR_VAR)
       PRINT 7, TRIM(carg)
     
    CASE (ERR_CMOR_WRITE)
       PRINT *, 'Error writing to CMOR output file'

    CASE (ERR_CMOR_CLOSE)
       PRINT *, 'Problems closing CMOR output file'

    CASE (ERR_NON_2D_FIXED)
       PRINT *, 'Non-2D time-independent data'

    CASE (ERR_UNKNOWN_AXIS)
       PRINT 8, carg

    CASE (ERR_SINGLETON_AXIS)
       PRINT *, 'Singleton axis out of sequence'

    CASE (ERR_UNKNOWN_VAR)
       PRINT 9, carg

    CASE (ERR_MODEL_FILE_OPEN)
       PRINT 10, TRIM(carg)

    CASE (ERR_MODEL_FILE_READ)
       PRINT 11, TRIM(carg)

    CASE (ERR_MODEL_FILE_CLOSE)
       PRINT *, 'Error closing model file'

    CASE (ERR_MODEL_INVALID_VAR)
       PRINT 12, TRIM(carg)

    CASE (ERR_UNSUPPORTED)
       PRINT 13, TRIM(carg)

    CASE (ERR_INTERNAL)
       PRINT 14, TRIM(carg)

    CASE (ERR_OUT_OF_MEMORY)
       PRINT *, 'ALLOCATE failed!'

    END SELECT

    STOP

1   FORMAT ('Invalid time range argument: ', A)
2   FORMAT ('Cannot open configuration file: ', A)
3   FORMAT ('Mandatory configuration variable not present: ', A)
4   FORMAT ('Invalid entry in configuration file (line ', I, ')')
5   FORMAT ('Duplicate key in configuration file (line ', I, ')')
6   FORMAT ('Error setting up CMOR axis: ', A)
7   FORMAT ('Error while setting up CMOR variable: ', A)
8   FORMAT ('Unknown axis: ', A)
9   FORMAT ('Unknown variable: ', A)
10  FORMAT ('Error opening model file: ', A)
11  FORMAT ('Error reading model file: ', A)
12  FORMAT ('Unsupported variable for model: ', A)
13  FORMAT ('Unsupported/unimplemented feature: ', A)
14  FORMAT ('INTERNAL ERROR: ', A)

  END SUBROUTINE fatal

END MODULE pp_utils

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
