!----------------------------------------------------------------------
!
!  File:   pp_vars.f90
!  Author: Ian Ross, University of Bristol
!  Date:   18 Feb 2005
!
!----------------------------------------------------------------------
!
!  Variable specific definitions for the MOTIF post-processor
!
!----------------------------------------------------------------------

MODULE pp_vars

  USE pp_defs
  USE pp_utils

  IMPLICIT NONE

  PRIVATE
  PUBLIC :: get_axes
  PUBLIC :: fill_pressure_axis
  PUBLIC :: motif_axis_units

CONTAINS

  !--------------------------------------------------------------------
  !
  !  Determine the axes used for a given MOTIF variable.
  !

  SUBROUTINE get_axes(var_name, freq, axes, axis_count)

    IMPLICIT NONE

    ! Variable name.
    CHARACTER(LEN=ARG_LEN),                     INTENT(IN)  :: var_name
    
    ! Output frequency (used for working out pressure levels for atmospheric
    ! variables).
    CHARACTER(LEN=2),                           INTENT(IN)  :: freq


    ! Count of axes.
    INTEGER,                                    INTENT(OUT) :: axis_count

    ! Required axes.
    CHARACTER(LEN=AXIS_NAME_LEN), DIMENSION(4), INTENT(OUT) :: axes


    CHARACTER(LEN=AXIS_NAME_LEN) :: tmp_axis


    SELECT CASE (var_name)

       ! Variables in more than one domain.

    CASE ('ts')

       axis_count = 3           ! XYT
       axes(1) = AXIS_LON; axes(2) = AXIS_LAT; axes(3) = AXIS_TIME


       ! Atmosphere variables.

    CASE ('orog', 'sftgif', 'sftlf')

       axis_count = 2           ! XY
       axes(1) = AXIS_LON; axes(2) = AXIS_LAT

    CASE ('albs', 'clivi', 'clt', 'clwvi', 'evspsbl', 'fpar', 'hfls', &
          'hfns', 'hfss', 'hurs', 'huss', 'intuadse', 'intuaw', 'intvadse', &
          'intvaw', 'pr', 'prc', 'prw', 'ps', 'psl', 'rlds', 'rldscs', 'rls', &
          'rlscs', 'rlus', 'rlut', 'rlutcs', 'rsds', 'rsdscs', 'rsdt', 'rss', &
          'rsscs', 'rst', 'rstcs', 'rsus', 'rsuscs', 'rsut', 'rsutcs', &
          'rtmt', 'sbls', 'snc', 'snd', 'snm')

       axis_count = 3           ! XYT
       axes(1) = AXIS_LON; axes(2) = AXIS_LAT; axes(3) = AXIS_TIME

    CASE ('tas', 'tasmax', 'tasmin')
       axis_count = 4           ! XYT + height marker 1 (2m)
       axes(1) = AXIS_LON;  axes(2) = AXIS_LAT
       axes(3) = AXIS_TIME; axes(4) = AXIS_HGT1

    CASE ('uas', 'uvas', 'vas')
       axis_count = 4           ! XYT + height marker 2 (10m)
       axes(1) = AXIS_LON;  axes(2) = AXIS_LAT
       axes(3) = AXIS_TIME; axes(4) = AXIS_HGT2

    CASE ('ta', 'cl', 'zg', 'hus', 'hur', 'ua', 'va', 'wap')

       CALL get_pressure_axis(var_name, freq, tmp_axis)
       IF (TRIM(tmp_axis) == AXIS_Pm .OR. TRIM(tmp_axis) == AXIS_Pb) THEN
          axis_count = 4        ! XYPT (singleton press. axis)
          axes(1) = AXIS_LON;  axes(2) = AXIS_LAT
          axes(3) = AXIS_TIME; axes(4) = tmp_axis
       ELSE
          axis_count = 4        ! XYPT
          axes(1) = AXIS_LON; axes(2) = AXIS_LAT
          axes(3) = tmp_axis; axes(4) = AXIS_TIME
       END IF

       ! Ocean variables.

    CASE ('sos', 'tos', 'zmlo', 'stfbarot', 'uos', 'vos', 'zos', 'hfo', &
          'wfo', 'tauu', 'tauv', 'hcont300', 'ziso20')

       axis_count = 3           ! XYT
       axes(1) = AXIS_LON; axes(2) = AXIS_LAT; axes(3) = AXIS_TIME

    CASE ('thetao', 'so', 'uo', 'vo', 'wo', 'mpuoeiso', 'mpuoeito', &
          'mpuoso', 'mpuoto', 'mpvoeiso', 'mpvoeito', 'mpvoso', 'mpvoto')

       axis_count = 4           ! XYZT
       axes(1) = AXIS_LON;   axes(2) = AXIS_LAT;
       axes(3) = AXIS_DEPTH; axes(4) = AXIS_TIME

    CASE ('sozm', 'thetaozm', 'stfmmc')

       axis_count = 4           ! YZRT
       axes(1) = AXIS_LAT;    axes(2) = AXIS_DEPTH;
       axes(3) = AXIS_REGION; axes(4) = AXIS_TIME

    CASE ('hfogo', 'sltov')

       axis_count = 3           ! YRT
       axes(1) = AXIS_LAT; axes(2) = AXIS_REGION; axes(3) = AXIS_TIME


       ! Surface variables.

    CASE ('vcarcont', 'scarcont', 'npp', 'bioburnt', 'lcarcont', 'cheight', &
          'gpp', 'hresp', 'mrro', 'mrros', 'mrso', 'dis', 'evspsblveg',     &
          'trans', 'esoil', 'potevap')

       axis_count = 3           ! XYT
       axes(1) = AXIS_LON; axes(2) = AXIS_LAT; axes(3) = AXIS_TIME

    CASE ('gpppft', 'laipft', 'lcarcontpft', 'npppft', 'pftfrac', &
          'potevappft', 'transpft', 'vcarcontpft')

       axis_count = 4           ! XYVT
       axes(1) = AXIS_LON; axes(2) = AXIS_LAT;
       axes(3) = AXIS_PFT; axes(4) = AXIS_TIME

    CASE ('tsoil')

       axis_count = 4           ! XYST
       axes(1) = AXIS_LON;   axes(2) = AXIS_LAT;
       axes(3) = AXIS_DEPTH; axes(4) = AXIS_TIME


       ! Ice variables.

    CASE ('sic', 'sit', 'usi', 'vsi', 'siprod', 'hfsib')

       axis_count = 3           ! XYT
       axes(1) = AXIS_LON; axes(2) = AXIS_LAT; axes(3) = AXIS_TIME

    CASE ('siarea', 'siext', 'sildarea', 'sisnvol', 'sivol')

       axis_count = 2           ! HT
       axes(1) = AXIS_HEMI; axes(2) = AXIS_TIME

    CASE ('sifram')

       axis_count = 1           ! T
       axes(1) = AXIS_TIME


       ! Unknown variable.

    CASE DEFAULT

       CALL fatal(ERR_UNKNOWN_VAR, var_name)

    END SELECT

  END SUBROUTINE get_axes


  !--------------------------------------------------------------------
  !
  !  Determine the pressure axis used for a given MOTIF atmospheric
  !  variable at a given output frequency.
  !

  SUBROUTINE get_pressure_axis(var_name, freq, axis)

    IMPLICIT NONE


    ! Variable name.
    CHARACTER(LEN=ARG_LEN),       INTENT(IN)  :: var_name
    
    ! Output frequency.
    CHARACTER(LEN=2),             INTENT(IN)  :: freq


    ! Required axis.
    CHARACTER(LEN=AXIS_NAME_LEN), INTENT(OUT) :: axis


    axis = ''

    SELECT CASE (var_name)

    CASE ('ta')
       SELECT CASE (freq)
       CASE (FREQ_DAILY)
          axis = AXIS_Pbm
       CASE (FREQ_MONTHLY)
          axis = AXIS_P3
       CASE (FREQ_SEASONAL)
          axis = AXIS_P17
       END SELECT

    CASE ('cl')
       IF (freq == FREQ_SEASONAL) axis = AXIS_P17

    CASE ('zg')
       SELECT CASE (freq)
       CASE (FREQ_DAILY)
          axis = AXIS_Pm
       CASE (FREQ_MONTHLY)
          axis = AXIS_P3
       CASE (FREQ_SEASONAL)
          axis = AXIS_P17
       END SELECT

    CASE ('hus')
       SELECT CASE (freq)
       CASE (FREQ_DAILY)
          axis = AXIS_Pb
       CASE (FREQ_MONTHLY)
          axis = AXIS_P3
       CASE (FREQ_SEASONAL)
          axis = AXIS_P17
       END SELECT

    CASE ('hur')
       IF (freq == FREQ_SEASONAL) axis = AXIS_P17

    CASE ('ua', 'va')
       SELECT CASE (freq)
       CASE (FREQ_DAILY)
          axis = AXIS_Pbt
       CASE (FREQ_MONTHLY)
          axis = AXIS_P3
       CASE (FREQ_SEASONAL)
          axis = AXIS_P17
       END SELECT

    CASE ('wap')
       SELECT CASE (freq)
       CASE (FREQ_DAILY)
          axis = AXIS_Pm
       CASE (FREQ_MONTHLY)
          axis = AXIS_P3
       CASE (FREQ_SEASONAL)
          axis = AXIS_P17
       END SELECT

    END SELECT

  END SUBROUTINE get_pressure_axis


  SUBROUTINE fill_pressure_axis(axis)

    IMPLICIT NONE

    ! Required axis.
    TYPE(axis_def), INTENT(INOUT) :: axis

    INTEGER :: alloc_stat

    SELECT CASE (axis%name)

    CASE (AXIS_P17)
       ALLOCATE(axis%values(17), STAT=alloc_stat)
       IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
       axis%values = 100.0 * (/ 1000,  925,  850,  700,  600,  500, &
                                 400,  300,  250,  200,  150,  100, &
                                  70,   50,   30,   20,   10 /)

    CASE (AXIS_P3)
       ALLOCATE(axis%values(3), STAT=alloc_stat)
       IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
       axis%values = 100.0 * (/ 850, 500, 200 /)

    CASE (AXIS_Pbt)
       ALLOCATE(axis%values(2), STAT=alloc_stat)
       IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
       axis%values = 100.0 * (/ 850, 200 /)

    CASE (AXIS_Pbm)
       ALLOCATE(axis%values(2), STAT=alloc_stat)
       IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
       axis%values = 100.0 * (/ 850, 500 /)

    CASE (AXIS_Pb)
       ALLOCATE(axis%values(1), STAT=alloc_stat)
       IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
       axis%values = 100.0 * (/ 850 /)

    CASE (AXIS_Pm)
       ALLOCATE(axis%values(1), STAT=alloc_stat)
       IF (alloc_stat /= 0) CALL fatal(ERR_OUT_OF_MEMORY)
       axis%values = 100.0 * (/ 500 /)

    END SELECT

!    ALLOCATE(axis%bounds(2, SIZE(axis%values)))

  END SUBROUTINE fill_pressure_axis


  FUNCTION motif_axis_units(axis) RESULT(motif_units)

    IMPLICIT NONE

    ! Required axis.
    CHARACTER(LEN=AXIS_NAME_LEN), INTENT(IN) :: axis

    CHARACTER(LEN=UNIT_NAME_LEN)             :: motif_units


    motif_units = ''
    SELECT CASE (axis)
    CASE (AXIS_LON)
       motif_units = 'degrees_east'
    CASE (AXIS_LAT)
       motif_units = 'degrees_north'
    CASE (AXIS_DEPTH, AXIS_HGT1, AXIS_HGT2)
       motif_units = 'm'
    CASE (AXIS_P17, AXIS_P3, AXIS_Pbt, AXIS_Pbm, AXIS_Pb, AXIS_Pm)
       motif_units = 'Pa'
    CASE (AXIS_PFT)
       motif_units = '1'
    CASE (AXIS_REGION, AXIS_HEMI)
       motif_units = ''
    CASE (AXIS_TIME)
       motif_units = 'days since 0000-01-01 00:00:00'
    END SELECT

  END FUNCTION motif_axis_units

END MODULE pp_vars

!----------------------------------------------------------------------
!----------------------------------------------------------------------
!----------------------------------------------------------------------
